# Copyright 2019 Shift Cryptosecurity AG
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""secp256k1 util functions"""
import hashlib

import ecdsa


class ECDSANonceException(Exception):
    pass


def tagged_sha256(tag: bytes, msg: bytes) -> bytes:
    tag_hash = hashlib.sha256(tag).digest()
    return hashlib.sha256(tag_hash + tag_hash + msg).digest()


def antiklepto_host_commit(host_nonce: bytes) -> bytes:
    return tagged_sha256(b"s2c/ecdsa/data", host_nonce)


def antiklepto_verify(host_nonce: bytes, signer_commitment: bytes, signature: bytes) -> None:
    """
    Verifies that hostNonce was used to tweak the nonce during signature
    generation according to k' = k + H(signerCommitment, hostNonce) by checking that
    k'*G = signerCommitment + H(signerCommitment, hostNonce)*G.
    Throws ECDSANonceException if the verification fails.
    """
    assert len(host_nonce) == 32
    assert len(signer_commitment) == 33, "expected compressed pubkey"
    assert len(signature) == 64
    signer_commitment_pubkey = ecdsa.VerifyingKey.from_string(
        signer_commitment, ecdsa.curves.SECP256k1
    )
    # Compute R = R1 + H(R1, host_nonce)*G. R1 is the client nonce commitment.
    tweak = tagged_sha256(b"s2c/ecdsa/point", signer_commitment + host_nonce)
    tweak_pubkey = ecdsa.SigningKey.from_string(tweak, curve=ecdsa.curves.SECP256k1).verifying_key
    tweaked_nonce = tweak_pubkey.pubkey.point + signer_commitment_pubkey.pubkey.point
    expected_sig_r = tweaked_nonce.x() % ecdsa.curves.SECP256k1.order
    sig_r = int.from_bytes(signature[:32], "big")
    if sig_r != expected_sig_r:
        raise ECDSANonceException(
            "Could not verify that the host nonce was contributed to the signature. "
            "If this happens repeatedly, the device might be attempting to "
            "leak the seed through the signature."
        )




"""Ledger Nano Bitcoin app client"""

from .client_base import Client, TransportClient
from .client import createClient
from ...common import Chain

from .wallet import AddressType, WalletPolicy, MultisigWallet

__all__ = ["Client", "TransportClient", "createClient", "Chain", "AddressType", "WalletPolicy", "MultisigWallet"]



# Copyright 2019 Shift Cryptosecurity AG
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""USB device utility functions"""

import re
from typing import List
from typing_extensions import TypedDict

import hid
import semver


BB02MULTI_BOOTLOADER = "bb02-bootloader"
BB02BTC_BOOTLOADER = "bb02btc-bootloader"
BITBOX02MULTI = "BitBox02"
BITBOX02BTC = "BitBox02BTC"

BITBOX02PLUS_MULTI_BOOTLOADER = "BitBox02 Nova Multi bl"
BITBOX02PLUS_BTC_BOOTLOADER = "BitBox02 Nova BTC-only bl"
BITBOX02PLUS_MULTI = "BitBox02 Nova Multi"
BITBOX02PLUS_BTC = "BitBox02 Nova BTC-only"


class TooManyFoundException(Exception):
    def __init__(self, count: int) -> None:
        super().__init__(f"Found {count} devices")


class NoneFoundException(Exception):
    def __init__(self) -> None:
        super().__init__("Found 0 devices")


class DeviceInfo(TypedDict):
    serial_number: str
    path: bytes
    product_string: str


def get_devices(product_string: str, interface_number: int = 0) -> List[DeviceInfo]:
    """
    Scans devices and returns a list of hid device info objects.
    """
    # TODO: product id is 0x2403, but 0x2402 is the id of some dev
    # device bootloaders. Can be removed in time, not needed for
    # production devices.
    # HWW/General endpoint is on interface 0
    # U2F Endpoint is on interface 1
    return [
        info
        for info in hid.enumerate()
        if info["vendor_id"] == 0x03EB
        and info["product_id"] in (0x2402, 0x2403)
        and (info["usage_page"] == 0xFFFF or info["interface_number"] == interface_number)
        and info["product_string"] == product_string
    ]


def get_device(product_string: str, interface_number: int = 0) -> DeviceInfo:
    devices = get_devices(product_string, interface_number)
    if len(devices) > 1:
        raise TooManyFoundException(len(devices))
    if not devices:
        raise NoneFoundException()
    return devices[0]


def get_bitbox02multi_device() -> DeviceInfo:
    return get_device(BITBOX02MULTI)


def get_bitbox02multi_bootloader() -> DeviceInfo:
    return get_device(BB02MULTI_BOOTLOADER)


def get_bitbox02btc_device() -> DeviceInfo:
    return get_device(BITBOX02BTC)


def get_bitbox02btc_bootloader() -> DeviceInfo:
    return get_device(BB02BTC_BOOTLOADER)


def get_bitbox02multi_devices() -> List[DeviceInfo]:
    return get_devices(BITBOX02MULTI)


def get_bitbox02multi_bootloaders() -> List[DeviceInfo]:
    return get_devices(BB02MULTI_BOOTLOADER)


def get_bitbox02btc_devices() -> List[DeviceInfo]:
    return get_devices(BITBOX02BTC)


def get_bitbox02btc_bootloaders() -> List[DeviceInfo]:
    return get_devices(BB02BTC_BOOTLOADER)


def get_any_bitbox02s() -> List[DeviceInfo]:
    """
    Searches for both btc-only and non-btc-only devices
    Returns:
        List of devices
    """
    devices = get_bitbox02multi_devices()
    devices.extend(get_bitbox02btc_devices())
    devices.extend(get_devices(BITBOX02PLUS_MULTI))
    devices.extend(get_devices(BITBOX02PLUS_BTC))
    return devices


def get_any_bitbox02() -> DeviceInfo:
    """
    Searches for both btc-only and non-btc-only devices
    Raises:
        TooManyFoundException: If more than 1 is found
    Returns:
        A device if found, otherwise None
    """
    devices = get_any_bitbox02s()
    if len(devices) > 1:
        raise TooManyFoundException(len(devices))
    if not devices:
        raise NoneFoundException()
    return devices[0]


def get_any_bitbox02_bootloaders() -> List[DeviceInfo]:
    """
    Searches for both btc-only and non-btc-only devices
    Returns:
        List of devices
    """
    devices = get_bitbox02multi_bootloaders()
    devices.extend(get_bitbox02btc_bootloaders())
    devices.extend(get_devices(BITBOX02PLUS_MULTI_BOOTLOADER))
    devices.extend(get_devices(BITBOX02PLUS_BTC_BOOTLOADER))
    return devices


def get_any_bitbox02_bootloader() -> DeviceInfo:
    """
    Searches for both btc-only and non-btc-only devices
    Raises:
        TooManyFoundException: If more than 1 is found
    Returns:
        A device if found, otherwise None
    """
    devices = get_any_bitbox02_bootloaders()
    if len(devices) > 1:
        raise TooManyFoundException(len(devices))
    if not devices:
        raise NoneFoundException
    return devices[0]


def parse_device_version(version: str) -> semver.VersionInfo:
    match = re.search(r"v([0-9]+\.[0-9]+\.[0-9]+.*)", version)
    if match is None:
        raise ValueError(f"Could not parse version string from string: {version}")

    return semver.VersionInfo.parse(match.group(1))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
This file is named backup_commands to avoid conflicting header files with top-most backup.proto"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CheckBackupRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SILENT_FIELD_NUMBER: builtins.int
    silent: builtins.bool
    def __init__(
        self,
        *,
        silent: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["silent", b"silent"]) -> None: ...

global___CheckBackupRequest = CheckBackupRequest

@typing.final
class CheckBackupResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id"]) -> None: ...

global___CheckBackupResponse = CheckBackupResponse

@typing.final
class CreateBackupRequest(google.protobuf.message.Message):
    """Timestamp must be in UTC"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: builtins.int
    timestamp: builtins.int
    timezone_offset: builtins.int
    def __init__(
        self,
        *,
        timestamp: builtins.int = ...,
        timezone_offset: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["timestamp", b"timestamp", "timezone_offset", b"timezone_offset"]) -> None: ...

global___CreateBackupRequest = CreateBackupRequest

@typing.final
class ListBackupsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ListBackupsRequest = ListBackupsRequest

@typing.final
class BackupInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    id: builtins.str
    timestamp: builtins.int
    name: builtins.str
    """uint32 timezone_offset = 3;"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        timestamp: builtins.int = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id", "name", b"name", "timestamp", b"timestamp"]) -> None: ...

global___BackupInfo = BackupInfo

@typing.final
class ListBackupsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFO_FIELD_NUMBER: builtins.int
    @property
    def info(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BackupInfo]: ...
    def __init__(
        self,
        *,
        info: collections.abc.Iterable[global___BackupInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["info", b"info"]) -> None: ...

global___ListBackupsResponse = ListBackupsResponse

@typing.final
class RestoreBackupRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: builtins.int
    id: builtins.str
    timestamp: builtins.int
    timezone_offset: builtins.int
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        timestamp: builtins.int = ...,
        timezone_offset: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id", "timestamp", b"timestamp", "timezone_offset", b"timezone_offset"]) -> None: ...

global___RestoreBackupRequest = RestoreBackupRequest



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2019 Shift Cryptosecurity AG

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from . import backup_commands_pb2
from . import bitbox02_system_pb2
from . import bluetooth_pb2
from . import btc_pb2
import builtins
from . import cardano_pb2
from . import common_pb2
from . import eth_pb2
import google.protobuf.descriptor
import google.protobuf.message
from . import keystore_pb2
from . import mnemonic_pb2
from . import perform_attestation_pb2
from . import system_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Error(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    code: builtins.int
    message: builtins.str
    def __init__(
        self,
        *,
        code: builtins.int = ...,
        message: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["code", b"code", "message", b"message"]) -> None: ...

global___Error = Error

@typing.final
class Success(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Success = Success

@typing.final
class Request(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEVICE_NAME_FIELD_NUMBER: builtins.int
    DEVICE_LANGUAGE_FIELD_NUMBER: builtins.int
    DEVICE_INFO_FIELD_NUMBER: builtins.int
    SET_PASSWORD_FIELD_NUMBER: builtins.int
    CREATE_BACKUP_FIELD_NUMBER: builtins.int
    SHOW_MNEMONIC_FIELD_NUMBER: builtins.int
    BTC_PUB_FIELD_NUMBER: builtins.int
    BTC_SIGN_INIT_FIELD_NUMBER: builtins.int
    BTC_SIGN_INPUT_FIELD_NUMBER: builtins.int
    BTC_SIGN_OUTPUT_FIELD_NUMBER: builtins.int
    INSERT_REMOVE_SDCARD_FIELD_NUMBER: builtins.int
    CHECK_SDCARD_FIELD_NUMBER: builtins.int
    SET_MNEMONIC_PASSPHRASE_ENABLED_FIELD_NUMBER: builtins.int
    LIST_BACKUPS_FIELD_NUMBER: builtins.int
    RESTORE_BACKUP_FIELD_NUMBER: builtins.int
    PERFORM_ATTESTATION_FIELD_NUMBER: builtins.int
    REBOOT_FIELD_NUMBER: builtins.int
    CHECK_BACKUP_FIELD_NUMBER: builtins.int
    ETH_FIELD_NUMBER: builtins.int
    RESET_FIELD_NUMBER: builtins.int
    RESTORE_FROM_MNEMONIC_FIELD_NUMBER: builtins.int
    FINGERPRINT_FIELD_NUMBER: builtins.int
    BTC_FIELD_NUMBER: builtins.int
    ELECTRUM_ENCRYPTION_KEY_FIELD_NUMBER: builtins.int
    CARDANO_FIELD_NUMBER: builtins.int
    BIP85_FIELD_NUMBER: builtins.int
    BLUETOOTH_FIELD_NUMBER: builtins.int
    @property
    def device_name(self) -> bitbox02_system_pb2.SetDeviceNameRequest:
        """removed: RandomNumberRequest random_number = 1;"""

    @property
    def device_language(self) -> bitbox02_system_pb2.SetDeviceLanguageRequest: ...
    @property
    def device_info(self) -> bitbox02_system_pb2.DeviceInfoRequest: ...
    @property
    def set_password(self) -> bitbox02_system_pb2.SetPasswordRequest: ...
    @property
    def create_backup(self) -> backup_commands_pb2.CreateBackupRequest: ...
    @property
    def show_mnemonic(self) -> mnemonic_pb2.ShowMnemonicRequest: ...
    @property
    def btc_pub(self) -> btc_pb2.BTCPubRequest: ...
    @property
    def btc_sign_init(self) -> btc_pb2.BTCSignInitRequest: ...
    @property
    def btc_sign_input(self) -> btc_pb2.BTCSignInputRequest: ...
    @property
    def btc_sign_output(self) -> btc_pb2.BTCSignOutputRequest: ...
    @property
    def insert_remove_sdcard(self) -> bitbox02_system_pb2.InsertRemoveSDCardRequest: ...
    @property
    def check_sdcard(self) -> bitbox02_system_pb2.CheckSDCardRequest: ...
    @property
    def set_mnemonic_passphrase_enabled(self) -> mnemonic_pb2.SetMnemonicPassphraseEnabledRequest: ...
    @property
    def list_backups(self) -> backup_commands_pb2.ListBackupsRequest: ...
    @property
    def restore_backup(self) -> backup_commands_pb2.RestoreBackupRequest: ...
    @property
    def perform_attestation(self) -> perform_attestation_pb2.PerformAttestationRequest: ...
    @property
    def reboot(self) -> system_pb2.RebootRequest: ...
    @property
    def check_backup(self) -> backup_commands_pb2.CheckBackupRequest: ...
    @property
    def eth(self) -> eth_pb2.ETHRequest: ...
    @property
    def reset(self) -> bitbox02_system_pb2.ResetRequest: ...
    @property
    def restore_from_mnemonic(self) -> mnemonic_pb2.RestoreFromMnemonicRequest: ...
    @property
    def fingerprint(self) -> common_pb2.RootFingerprintRequest:
        """removed: BitBoxBaseRequest bitboxbase = 23;"""

    @property
    def btc(self) -> btc_pb2.BTCRequest: ...
    @property
    def electrum_encryption_key(self) -> keystore_pb2.ElectrumEncryptionKeyRequest: ...
    @property
    def cardano(self) -> cardano_pb2.CardanoRequest: ...
    @property
    def bip85(self) -> keystore_pb2.BIP85Request: ...
    @property
    def bluetooth(self) -> bluetooth_pb2.BluetoothRequest: ...
    def __init__(
        self,
        *,
        device_name: bitbox02_system_pb2.SetDeviceNameRequest | None = ...,
        device_language: bitbox02_system_pb2.SetDeviceLanguageRequest | None = ...,
        device_info: bitbox02_system_pb2.DeviceInfoRequest | None = ...,
        set_password: bitbox02_system_pb2.SetPasswordRequest | None = ...,
        create_backup: backup_commands_pb2.CreateBackupRequest | None = ...,
        show_mnemonic: mnemonic_pb2.ShowMnemonicRequest | None = ...,
        btc_pub: btc_pb2.BTCPubRequest | None = ...,
        btc_sign_init: btc_pb2.BTCSignInitRequest | None = ...,
        btc_sign_input: btc_pb2.BTCSignInputRequest | None = ...,
        btc_sign_output: btc_pb2.BTCSignOutputRequest | None = ...,
        insert_remove_sdcard: bitbox02_system_pb2.InsertRemoveSDCardRequest | None = ...,
        check_sdcard: bitbox02_system_pb2.CheckSDCardRequest | None = ...,
        set_mnemonic_passphrase_enabled: mnemonic_pb2.SetMnemonicPassphraseEnabledRequest | None = ...,
        list_backups: backup_commands_pb2.ListBackupsRequest | None = ...,
        restore_backup: backup_commands_pb2.RestoreBackupRequest | None = ...,
        perform_attestation: perform_attestation_pb2.PerformAttestationRequest | None = ...,
        reboot: system_pb2.RebootRequest | None = ...,
        check_backup: backup_commands_pb2.CheckBackupRequest | None = ...,
        eth: eth_pb2.ETHRequest | None = ...,
        reset: bitbox02_system_pb2.ResetRequest | None = ...,
        restore_from_mnemonic: mnemonic_pb2.RestoreFromMnemonicRequest | None = ...,
        fingerprint: common_pb2.RootFingerprintRequest | None = ...,
        btc: btc_pb2.BTCRequest | None = ...,
        electrum_encryption_key: keystore_pb2.ElectrumEncryptionKeyRequest | None = ...,
        cardano: cardano_pb2.CardanoRequest | None = ...,
        bip85: keystore_pb2.BIP85Request | None = ...,
        bluetooth: bluetooth_pb2.BluetoothRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["bip85", b"bip85", "bluetooth", b"bluetooth", "btc", b"btc", "btc_pub", b"btc_pub", "btc_sign_init", b"btc_sign_init", "btc_sign_input", b"btc_sign_input", "btc_sign_output", b"btc_sign_output", "cardano", b"cardano", "check_backup", b"check_backup", "check_sdcard", b"check_sdcard", "create_backup", b"create_backup", "device_info", b"device_info", "device_language", b"device_language", "device_name", b"device_name", "electrum_encryption_key", b"electrum_encryption_key", "eth", b"eth", "fingerprint", b"fingerprint", "insert_remove_sdcard", b"insert_remove_sdcard", "list_backups", b"list_backups", "perform_attestation", b"perform_attestation", "reboot", b"reboot", "request", b"request", "reset", b"reset", "restore_backup", b"restore_backup", "restore_from_mnemonic", b"restore_from_mnemonic", "set_mnemonic_passphrase_enabled", b"set_mnemonic_passphrase_enabled", "set_password", b"set_password", "show_mnemonic", b"show_mnemonic"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["bip85", b"bip85", "bluetooth", b"bluetooth", "btc", b"btc", "btc_pub", b"btc_pub", "btc_sign_init", b"btc_sign_init", "btc_sign_input", b"btc_sign_input", "btc_sign_output", b"btc_sign_output", "cardano", b"cardano", "check_backup", b"check_backup", "check_sdcard", b"check_sdcard", "create_backup", b"create_backup", "device_info", b"device_info", "device_language", b"device_language", "device_name", b"device_name", "electrum_encryption_key", b"electrum_encryption_key", "eth", b"eth", "fingerprint", b"fingerprint", "insert_remove_sdcard", b"insert_remove_sdcard", "list_backups", b"list_backups", "perform_attestation", b"perform_attestation", "reboot", b"reboot", "request", b"request", "reset", b"reset", "restore_backup", b"restore_backup", "restore_from_mnemonic", b"restore_from_mnemonic", "set_mnemonic_passphrase_enabled", b"set_mnemonic_passphrase_enabled", "set_password", b"set_password", "show_mnemonic", b"show_mnemonic"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["request", b"request"]) -> typing.Literal["device_name", "device_language", "device_info", "set_password", "create_backup", "show_mnemonic", "btc_pub", "btc_sign_init", "btc_sign_input", "btc_sign_output", "insert_remove_sdcard", "check_sdcard", "set_mnemonic_passphrase_enabled", "list_backups", "restore_backup", "perform_attestation", "reboot", "check_backup", "eth", "reset", "restore_from_mnemonic", "fingerprint", "btc", "electrum_encryption_key", "cardano", "bip85", "bluetooth"] | None: ...

global___Request = Request

@typing.final
class Response(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    DEVICE_INFO_FIELD_NUMBER: builtins.int
    PUB_FIELD_NUMBER: builtins.int
    BTC_SIGN_NEXT_FIELD_NUMBER: builtins.int
    LIST_BACKUPS_FIELD_NUMBER: builtins.int
    CHECK_BACKUP_FIELD_NUMBER: builtins.int
    PERFORM_ATTESTATION_FIELD_NUMBER: builtins.int
    CHECK_SDCARD_FIELD_NUMBER: builtins.int
    ETH_FIELD_NUMBER: builtins.int
    FINGERPRINT_FIELD_NUMBER: builtins.int
    BTC_FIELD_NUMBER: builtins.int
    ELECTRUM_ENCRYPTION_KEY_FIELD_NUMBER: builtins.int
    CARDANO_FIELD_NUMBER: builtins.int
    BIP85_FIELD_NUMBER: builtins.int
    BLUETOOTH_FIELD_NUMBER: builtins.int
    @property
    def success(self) -> global___Success: ...
    @property
    def error(self) -> global___Error: ...
    @property
    def device_info(self) -> bitbox02_system_pb2.DeviceInfoResponse:
        """removed: RandomNumberResponse random_number = 3;"""

    @property
    def pub(self) -> common_pb2.PubResponse: ...
    @property
    def btc_sign_next(self) -> btc_pb2.BTCSignNextResponse: ...
    @property
    def list_backups(self) -> backup_commands_pb2.ListBackupsResponse: ...
    @property
    def check_backup(self) -> backup_commands_pb2.CheckBackupResponse: ...
    @property
    def perform_attestation(self) -> perform_attestation_pb2.PerformAttestationResponse: ...
    @property
    def check_sdcard(self) -> bitbox02_system_pb2.CheckSDCardResponse: ...
    @property
    def eth(self) -> eth_pb2.ETHResponse: ...
    @property
    def fingerprint(self) -> common_pb2.RootFingerprintResponse: ...
    @property
    def btc(self) -> btc_pb2.BTCResponse: ...
    @property
    def electrum_encryption_key(self) -> keystore_pb2.ElectrumEncryptionKeyResponse: ...
    @property
    def cardano(self) -> cardano_pb2.CardanoResponse: ...
    @property
    def bip85(self) -> keystore_pb2.BIP85Response: ...
    @property
    def bluetooth(self) -> bluetooth_pb2.BluetoothResponse: ...
    def __init__(
        self,
        *,
        success: global___Success | None = ...,
        error: global___Error | None = ...,
        device_info: bitbox02_system_pb2.DeviceInfoResponse | None = ...,
        pub: common_pb2.PubResponse | None = ...,
        btc_sign_next: btc_pb2.BTCSignNextResponse | None = ...,
        list_backups: backup_commands_pb2.ListBackupsResponse | None = ...,
        check_backup: backup_commands_pb2.CheckBackupResponse | None = ...,
        perform_attestation: perform_attestation_pb2.PerformAttestationResponse | None = ...,
        check_sdcard: bitbox02_system_pb2.CheckSDCardResponse | None = ...,
        eth: eth_pb2.ETHResponse | None = ...,
        fingerprint: common_pb2.RootFingerprintResponse | None = ...,
        btc: btc_pb2.BTCResponse | None = ...,
        electrum_encryption_key: keystore_pb2.ElectrumEncryptionKeyResponse | None = ...,
        cardano: cardano_pb2.CardanoResponse | None = ...,
        bip85: keystore_pb2.BIP85Response | None = ...,
        bluetooth: bluetooth_pb2.BluetoothResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["bip85", b"bip85", "bluetooth", b"bluetooth", "btc", b"btc", "btc_sign_next", b"btc_sign_next", "cardano", b"cardano", "check_backup", b"check_backup", "check_sdcard", b"check_sdcard", "device_info", b"device_info", "electrum_encryption_key", b"electrum_encryption_key", "error", b"error", "eth", b"eth", "fingerprint", b"fingerprint", "list_backups", b"list_backups", "perform_attestation", b"perform_attestation", "pub", b"pub", "response", b"response", "success", b"success"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["bip85", b"bip85", "bluetooth", b"bluetooth", "btc", b"btc", "btc_sign_next", b"btc_sign_next", "cardano", b"cardano", "check_backup", b"check_backup", "check_sdcard", b"check_sdcard", "device_info", b"device_info", "electrum_encryption_key", b"electrum_encryption_key", "error", b"error", "eth", b"eth", "fingerprint", b"fingerprint", "list_backups", b"list_backups", "perform_attestation", b"perform_attestation", "pub", b"pub", "response", b"response", "success", b"success"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["response", b"response"]) -> typing.Literal["success", "error", "device_info", "pub", "btc_sign_next", "list_backups", "check_backup", "perform_attestation", "check_sdcard", "eth", "fingerprint", "btc", "electrum_encryption_key", "cardano", "bip85", "bluetooth"] | None: ...

global___Response = Response



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'signpsbtdialog.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_SignPSBTDialog(object):
    def setupUi(self, SignPSBTDialog):
        if not SignPSBTDialog.objectName():
            SignPSBTDialog.setObjectName(u"SignPSBTDialog")
        SignPSBTDialog.resize(650, 400)
        self.gridLayout = QGridLayout(SignPSBTDialog)
        self.gridLayout.setSpacing(10)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalSpacer_2 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer_2)

        self.label = QLabel(SignPSBTDialog)
        self.label.setObjectName(u"label")
        self.label.setWordWrap(True)

        self.verticalLayout_2.addWidget(self.label)

        self.import_toolbutton = QToolButton(SignPSBTDialog)
        self.import_toolbutton.setObjectName(u"import_toolbutton")
        self.import_toolbutton.setPopupMode(QToolButton.InstantPopup)

        self.verticalLayout_2.addWidget(self.import_toolbutton)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)


        self.gridLayout.addLayout(self.verticalLayout_2, 1, 0, 1, 1)

        self.psbt_in_textedit = QPlainTextEdit(SignPSBTDialog)
        self.psbt_in_textedit.setObjectName(u"psbt_in_textedit")
        self.psbt_in_textedit.setMinimumSize(QSize(300, 120))

        self.gridLayout.addWidget(self.psbt_in_textedit, 1, 2, 1, 1)

        self.buttonBox = QDialogButtonBox(SignPSBTDialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Close)

        self.gridLayout.addWidget(self.buttonBox, 4, 2, 1, 1)

        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalSpacer_3 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer_3)

        self.label_2 = QLabel(SignPSBTDialog)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setWordWrap(True)

        self.verticalLayout.addWidget(self.label_2)

        self.export_toolbutton = QToolButton(SignPSBTDialog)
        self.export_toolbutton.setObjectName(u"export_toolbutton")
        self.export_toolbutton.setPopupMode(QToolButton.InstantPopup)

        self.verticalLayout.addWidget(self.export_toolbutton)

        self.verticalSpacer_4 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer_4)


        self.gridLayout.addLayout(self.verticalLayout, 3, 0, 1, 1)

        self.psbt_out_textedit = QPlainTextEdit(SignPSBTDialog)
        self.psbt_out_textedit.setObjectName(u"psbt_out_textedit")
        self.psbt_out_textedit.setMinimumSize(QSize(300, 120))
        self.psbt_out_textedit.setTextInteractionFlags(Qt.TextSelectableByKeyboard|Qt.TextSelectableByMouse)

        self.gridLayout.addWidget(self.psbt_out_textedit, 3, 2, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.sign_psbt_button = QPushButton(SignPSBTDialog)
        self.sign_psbt_button.setObjectName(u"sign_psbt_button")
        self.sign_psbt_button.setAutoDefault(False)

        self.horizontalLayout.addWidget(self.sign_psbt_button)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)


        self.gridLayout.addLayout(self.horizontalLayout, 2, 2, 1, 1)


        self.retranslateUi(SignPSBTDialog)
        self.buttonBox.accepted.connect(SignPSBTDialog.accept)
        self.buttonBox.rejected.connect(SignPSBTDialog.reject)

        self.sign_psbt_button.setDefault(True)


        QMetaObject.connectSlotsByName(SignPSBTDialog)
    # setupUi

    def retranslateUi(self, SignPSBTDialog):
        SignPSBTDialog.setWindowTitle(QCoreApplication.translate("SignPSBTDialog", u"Dialog", None))
        self.label.setText(QCoreApplication.translate("SignPSBTDialog", u"PSBT To Sign", None))
        self.import_toolbutton.setText(QCoreApplication.translate("SignPSBTDialog", u"Import PSBT", None))
        self.label_2.setText(QCoreApplication.translate("SignPSBTDialog", u"PSBT Result", None))
        self.export_toolbutton.setText(QCoreApplication.translate("SignPSBTDialog", u"Export PSBT", None))
        self.sign_psbt_button.setText(QCoreApplication.translate("SignPSBTDialog", u"Sign PSBT", None))
    # retranslateUi




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: btc.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from . import antiklepto_pb2 as antiklepto__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tbtc.proto\x12\x14shiftcrypto.bitbox02\x1a\x0c\x63ommon.proto\x1a\x10\x61ntiklepto.proto\"\xc6\x04\n\x0f\x42TCScriptConfig\x12G\n\x0bsimple_type\x18\x01 \x01(\x0e\x32\x30.shiftcrypto.bitbox02.BTCScriptConfig.SimpleTypeH\x00\x12\x42\n\x08multisig\x18\x02 \x01(\x0b\x32..shiftcrypto.bitbox02.BTCScriptConfig.MultisigH\x00\x12>\n\x06policy\x18\x03 \x01(\x0b\x32,.shiftcrypto.bitbox02.BTCScriptConfig.PolicyH\x00\x1a\xd9\x01\n\x08Multisig\x12\x11\n\tthreshold\x18\x01 \x01(\r\x12)\n\x05xpubs\x18\x02 \x03(\x0b\x32\x1a.shiftcrypto.bitbox02.XPub\x12\x16\n\x0eour_xpub_index\x18\x03 \x01(\r\x12N\n\x0bscript_type\x18\x04 \x01(\x0e\x32\x39.shiftcrypto.bitbox02.BTCScriptConfig.Multisig.ScriptType\"\'\n\nScriptType\x12\t\n\x05P2WSH\x10\x00\x12\x0e\n\nP2WSH_P2SH\x10\x01\x1aK\n\x06Policy\x12\x0e\n\x06policy\x18\x01 \x01(\t\x12\x31\n\x04keys\x18\x02 \x03(\x0b\x32#.shiftcrypto.bitbox02.KeyOriginInfo\"3\n\nSimpleType\x12\x0f\n\x0bP2WPKH_P2SH\x10\x00\x12\n\n\x06P2WPKH\x10\x01\x12\x08\n\x04P2TR\x10\x02\x42\x08\n\x06\x63onfig\"\xfc\x02\n\rBTCPubRequest\x12+\n\x04\x63oin\x18\x01 \x01(\x0e\x32\x1d.shiftcrypto.bitbox02.BTCCoin\x12\x0f\n\x07keypath\x18\x02 \x03(\r\x12\x41\n\txpub_type\x18\x03 \x01(\x0e\x32,.shiftcrypto.bitbox02.BTCPubRequest.XPubTypeH\x00\x12>\n\rscript_config\x18\x04 \x01(\x0b\x32%.shiftcrypto.bitbox02.BTCScriptConfigH\x00\x12\x0f\n\x07\x64isplay\x18\x05 \x01(\x08\"\x8e\x01\n\x08XPubType\x12\x08\n\x04TPUB\x10\x00\x12\x08\n\x04XPUB\x10\x01\x12\x08\n\x04YPUB\x10\x02\x12\x08\n\x04ZPUB\x10\x03\x12\x08\n\x04VPUB\x10\x04\x12\x08\n\x04UPUB\x10\x05\x12\x10\n\x0c\x43\x41PITAL_VPUB\x10\x06\x12\x10\n\x0c\x43\x41PITAL_ZPUB\x10\x07\x12\x10\n\x0c\x43\x41PITAL_UPUB\x10\x08\x12\x10\n\x0c\x43\x41PITAL_YPUB\x10\tB\x08\n\x06output\"k\n\x1a\x42TCScriptConfigWithKeypath\x12<\n\rscript_config\x18\x02 \x01(\x0b\x32%.shiftcrypto.bitbox02.BTCScriptConfig\x12\x0f\n\x07keypath\x18\x03 \x03(\r\"\xbf\x03\n\x12\x42TCSignInitRequest\x12+\n\x04\x63oin\x18\x01 \x01(\x0e\x32\x1d.shiftcrypto.bitbox02.BTCCoin\x12H\n\x0escript_configs\x18\x02 \x03(\x0b\x32\x30.shiftcrypto.bitbox02.BTCScriptConfigWithKeypath\x12\x0f\n\x07version\x18\x04 \x01(\r\x12\x12\n\nnum_inputs\x18\x05 \x01(\r\x12\x13\n\x0bnum_outputs\x18\x06 \x01(\r\x12\x10\n\x08locktime\x18\x07 \x01(\r\x12H\n\x0b\x66ormat_unit\x18\x08 \x01(\x0e\x32\x33.shiftcrypto.bitbox02.BTCSignInitRequest.FormatUnit\x12\'\n\x1f\x63ontains_silent_payment_outputs\x18\t \x01(\x08\x12O\n\x15output_script_configs\x18\n \x03(\x0b\x32\x30.shiftcrypto.bitbox02.BTCScriptConfigWithKeypath\"\"\n\nFormatUnit\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x07\n\x03SAT\x10\x01\"\xc4\x03\n\x13\x42TCSignNextResponse\x12<\n\x04type\x18\x01 \x01(\x0e\x32..shiftcrypto.bitbox02.BTCSignNextResponse.Type\x12\r\n\x05index\x18\x02 \x01(\r\x12\x15\n\rhas_signature\x18\x03 \x01(\x08\x12\x11\n\tsignature\x18\x04 \x01(\x0c\x12\x12\n\nprev_index\x18\x05 \x01(\r\x12W\n\x1d\x61nti_klepto_signer_commitment\x18\x06 \x01(\x0b\x32\x30.shiftcrypto.bitbox02.AntiKleptoSignerCommitment\x12!\n\x19generated_output_pkscript\x18\x07 \x01(\x0c\x12!\n\x19silent_payment_dleq_proof\x18\x08 \x01(\x0c\"\x82\x01\n\x04Type\x12\t\n\x05INPUT\x10\x00\x12\n\n\x06OUTPUT\x10\x01\x12\x08\n\x04\x44ONE\x10\x02\x12\x0f\n\x0bPREVTX_INIT\x10\x03\x12\x10\n\x0cPREVTX_INPUT\x10\x04\x12\x11\n\rPREVTX_OUTPUT\x10\x05\x12\x0e\n\nHOST_NONCE\x10\x06\x12\x13\n\x0fPAYMENT_REQUEST\x10\x07\"\xea\x01\n\x13\x42TCSignInputRequest\x12\x13\n\x0bprevOutHash\x18\x01 \x01(\x0c\x12\x14\n\x0cprevOutIndex\x18\x02 \x01(\r\x12\x14\n\x0cprevOutValue\x18\x03 \x01(\x04\x12\x10\n\x08sequence\x18\x04 \x01(\r\x12\x0f\n\x07keypath\x18\x06 \x03(\r\x12\x1b\n\x13script_config_index\x18\x07 \x01(\r\x12R\n\x15host_nonce_commitment\x18\x08 \x01(\x0b\x32\x33.shiftcrypto.bitbox02.AntiKleptoHostNonceCommitment\"\x9f\x03\n\x14\x42TCSignOutputRequest\x12\x0c\n\x04ours\x18\x01 \x01(\x08\x12\x31\n\x04type\x18\x02 \x01(\x0e\x32#.shiftcrypto.bitbox02.BTCOutputType\x12\r\n\x05value\x18\x03 \x01(\x04\x12\x0f\n\x07payload\x18\x04 \x01(\x0c\x12\x0f\n\x07keypath\x18\x05 \x03(\r\x12\x1b\n\x13script_config_index\x18\x06 \x01(\r\x12\"\n\x15payment_request_index\x18\x07 \x01(\rH\x00\x88\x01\x01\x12P\n\x0esilent_payment\x18\x08 \x01(\x0b\x32\x38.shiftcrypto.bitbox02.BTCSignOutputRequest.SilentPayment\x12\'\n\x1aoutput_script_config_index\x18\t \x01(\rH\x01\x88\x01\x01\x1a \n\rSilentPayment\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\n\x16_payment_request_indexB\x1d\n\x1b_output_script_config_index\"\x99\x01\n\x1b\x42TCScriptConfigRegistration\x12+\n\x04\x63oin\x18\x01 \x01(\x0e\x32\x1d.shiftcrypto.bitbox02.BTCCoin\x12<\n\rscript_config\x18\x02 \x01(\x0b\x32%.shiftcrypto.bitbox02.BTCScriptConfig\x12\x0f\n\x07keypath\x18\x03 \x03(\r\"\x0c\n\nBTCSuccess\"m\n\"BTCIsScriptConfigRegisteredRequest\x12G\n\x0cregistration\x18\x01 \x01(\x0b\x32\x31.shiftcrypto.bitbox02.BTCScriptConfigRegistration\"<\n#BTCIsScriptConfigRegisteredResponse\x12\x15\n\ris_registered\x18\x01 \x01(\x08\"\xfc\x01\n\x1e\x42TCRegisterScriptConfigRequest\x12G\n\x0cregistration\x18\x01 \x01(\x0b\x32\x31.shiftcrypto.bitbox02.BTCScriptConfigRegistration\x12\x0c\n\x04name\x18\x02 \x01(\t\x12P\n\txpub_type\x18\x03 \x01(\x0e\x32=.shiftcrypto.bitbox02.BTCRegisterScriptConfigRequest.XPubType\"1\n\x08XPubType\x12\x11\n\rAUTO_ELECTRUM\x10\x00\x12\x12\n\x0e\x41UTO_XPUB_TPUB\x10\x01\"b\n\x14\x42TCPrevTxInitRequest\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x12\n\nnum_inputs\x18\x02 \x01(\r\x12\x13\n\x0bnum_outputs\x18\x03 \x01(\r\x12\x10\n\x08locktime\x18\x04 \x01(\r\"r\n\x15\x42TCPrevTxInputRequest\x12\x15\n\rprev_out_hash\x18\x01 \x01(\x0c\x12\x16\n\x0eprev_out_index\x18\x02 \x01(\r\x12\x18\n\x10signature_script\x18\x03 \x01(\x0c\x12\x10\n\x08sequence\x18\x04 \x01(\r\">\n\x16\x42TCPrevTxOutputRequest\x12\r\n\x05value\x18\x01 \x01(\x04\x12\x15\n\rpubkey_script\x18\x02 \x01(\x0c\"\xab\x02\n\x18\x42TCPaymentRequestRequest\x12\x16\n\x0erecipient_name\x18\x01 \x01(\t\x12\x42\n\x05memos\x18\x02 \x03(\x0b\x32\x33.shiftcrypto.bitbox02.BTCPaymentRequestRequest.Memo\x12\r\n\x05nonce\x18\x03 \x01(\x0c\x12\x14\n\x0ctotal_amount\x18\x04 \x01(\x04\x12\x11\n\tsignature\x18\x05 \x01(\x0c\x1a{\n\x04Memo\x12Q\n\ttext_memo\x18\x01 \x01(\x0b\x32<.shiftcrypto.bitbox02.BTCPaymentRequestRequest.Memo.TextMemoH\x00\x1a\x18\n\x08TextMemo\x12\x0c\n\x04note\x18\x01 \x01(\tB\x06\n\x04memo\"\xee\x01\n\x15\x42TCSignMessageRequest\x12+\n\x04\x63oin\x18\x01 \x01(\x0e\x32\x1d.shiftcrypto.bitbox02.BTCCoin\x12G\n\rscript_config\x18\x02 \x01(\x0b\x32\x30.shiftcrypto.bitbox02.BTCScriptConfigWithKeypath\x12\x0b\n\x03msg\x18\x03 \x01(\x0c\x12R\n\x15host_nonce_commitment\x18\x04 \x01(\x0b\x32\x33.shiftcrypto.bitbox02.AntiKleptoHostNonceCommitment\"+\n\x16\x42TCSignMessageResponse\x12\x11\n\tsignature\x18\x01 \x01(\x0c\"\x81\x05\n\nBTCRequest\x12_\n\x1bis_script_config_registered\x18\x01 \x01(\x0b\x32\x38.shiftcrypto.bitbox02.BTCIsScriptConfigRegisteredRequestH\x00\x12V\n\x16register_script_config\x18\x02 \x01(\x0b\x32\x34.shiftcrypto.bitbox02.BTCRegisterScriptConfigRequestH\x00\x12\x41\n\x0bprevtx_init\x18\x03 \x01(\x0b\x32*.shiftcrypto.bitbox02.BTCPrevTxInitRequestH\x00\x12\x43\n\x0cprevtx_input\x18\x04 \x01(\x0b\x32+.shiftcrypto.bitbox02.BTCPrevTxInputRequestH\x00\x12\x45\n\rprevtx_output\x18\x05 \x01(\x0b\x32,.shiftcrypto.bitbox02.BTCPrevTxOutputRequestH\x00\x12\x43\n\x0csign_message\x18\x06 \x01(\x0b\x32+.shiftcrypto.bitbox02.BTCSignMessageRequestH\x00\x12P\n\x14\x61ntiklepto_signature\x18\x07 \x01(\x0b\x32\x30.shiftcrypto.bitbox02.AntiKleptoSignatureRequestH\x00\x12I\n\x0fpayment_request\x18\x08 \x01(\x0b\x32..shiftcrypto.bitbox02.BTCPaymentRequestRequestH\x00\x42\t\n\x07request\"\x90\x03\n\x0b\x42TCResponse\x12\x33\n\x07success\x18\x01 \x01(\x0b\x32 .shiftcrypto.bitbox02.BTCSuccessH\x00\x12`\n\x1bis_script_config_registered\x18\x02 \x01(\x0b\x32\x39.shiftcrypto.bitbox02.BTCIsScriptConfigRegisteredResponseH\x00\x12>\n\tsign_next\x18\x03 \x01(\x0b\x32).shiftcrypto.bitbox02.BTCSignNextResponseH\x00\x12\x44\n\x0csign_message\x18\x04 \x01(\x0b\x32,.shiftcrypto.bitbox02.BTCSignMessageResponseH\x00\x12X\n\x1c\x61ntiklepto_signer_commitment\x18\x05 \x01(\x0b\x32\x30.shiftcrypto.bitbox02.AntiKleptoSignerCommitmentH\x00\x42\n\n\x08response*9\n\x07\x42TCCoin\x12\x07\n\x03\x42TC\x10\x00\x12\x08\n\x04TBTC\x10\x01\x12\x07\n\x03LTC\x10\x02\x12\x08\n\x04TLTC\x10\x03\x12\x08\n\x04RBTC\x10\x04*R\n\rBTCOutputType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05P2PKH\x10\x01\x12\x08\n\x04P2SH\x10\x02\x12\n\n\x06P2WPKH\x10\x03\x12\t\n\x05P2WSH\x10\x04\x12\x08\n\x04P2TR\x10\x05\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'btc_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BTCCOIN._serialized_start=5217
  _BTCCOIN._serialized_end=5274
  _BTCOUTPUTTYPE._serialized_start=5276
  _BTCOUTPUTTYPE._serialized_end=5358
  _BTCSCRIPTCONFIG._serialized_start=68
  _BTCSCRIPTCONFIG._serialized_end=650
  _BTCSCRIPTCONFIG_MULTISIG._serialized_start=293
  _BTCSCRIPTCONFIG_MULTISIG._serialized_end=510
  _BTCSCRIPTCONFIG_MULTISIG_SCRIPTTYPE._serialized_start=471
  _BTCSCRIPTCONFIG_MULTISIG_SCRIPTTYPE._serialized_end=510
  _BTCSCRIPTCONFIG_POLICY._serialized_start=512
  _BTCSCRIPTCONFIG_POLICY._serialized_end=587
  _BTCSCRIPTCONFIG_SIMPLETYPE._serialized_start=589
  _BTCSCRIPTCONFIG_SIMPLETYPE._serialized_end=640
  _BTCPUBREQUEST._serialized_start=653
  _BTCPUBREQUEST._serialized_end=1033
  _BTCPUBREQUEST_XPUBTYPE._serialized_start=881
  _BTCPUBREQUEST_XPUBTYPE._serialized_end=1023
  _BTCSCRIPTCONFIGWITHKEYPATH._serialized_start=1035
  _BTCSCRIPTCONFIGWITHKEYPATH._serialized_end=1142
  _BTCSIGNINITREQUEST._serialized_start=1145
  _BTCSIGNINITREQUEST._serialized_end=1592
  _BTCSIGNINITREQUEST_FORMATUNIT._serialized_start=1558
  _BTCSIGNINITREQUEST_FORMATUNIT._serialized_end=1592
  _BTCSIGNNEXTRESPONSE._serialized_start=1595
  _BTCSIGNNEXTRESPONSE._serialized_end=2047
  _BTCSIGNNEXTRESPONSE_TYPE._serialized_start=1917
  _BTCSIGNNEXTRESPONSE_TYPE._serialized_end=2047
  _BTCSIGNINPUTREQUEST._serialized_start=2050
  _BTCSIGNINPUTREQUEST._serialized_end=2284
  _BTCSIGNOUTPUTREQUEST._serialized_start=2287
  _BTCSIGNOUTPUTREQUEST._serialized_end=2702
  _BTCSIGNOUTPUTREQUEST_SILENTPAYMENT._serialized_start=2613
  _BTCSIGNOUTPUTREQUEST_SILENTPAYMENT._serialized_end=2645
  _BTCSCRIPTCONFIGREGISTRATION._serialized_start=2705
  _BTCSCRIPTCONFIGREGISTRATION._serialized_end=2858
  _BTCSUCCESS._serialized_start=2860
  _BTCSUCCESS._serialized_end=2872
  _BTCISSCRIPTCONFIGREGISTEREDREQUEST._serialized_start=2874
  _BTCISSCRIPTCONFIGREGISTEREDREQUEST._serialized_end=2983
  _BTCISSCRIPTCONFIGREGISTEREDRESPONSE._serialized_start=2985
  _BTCISSCRIPTCONFIGREGISTEREDRESPONSE._serialized_end=3045
  _BTCREGISTERSCRIPTCONFIGREQUEST._serialized_start=3048
  _BTCREGISTERSCRIPTCONFIGREQUEST._serialized_end=3300
  _BTCREGISTERSCRIPTCONFIGREQUEST_XPUBTYPE._serialized_start=3251
  _BTCREGISTERSCRIPTCONFIGREQUEST_XPUBTYPE._serialized_end=3300
  _BTCPREVTXINITREQUEST._serialized_start=3302
  _BTCPREVTXINITREQUEST._serialized_end=3400
  _BTCPREVTXINPUTREQUEST._serialized_start=3402
  _BTCPREVTXINPUTREQUEST._serialized_end=3516
  _BTCPREVTXOUTPUTREQUEST._serialized_start=3518
  _BTCPREVTXOUTPUTREQUEST._serialized_end=3580
  _BTCPAYMENTREQUESTREQUEST._serialized_start=3583
  _BTCPAYMENTREQUESTREQUEST._serialized_end=3882
  _BTCPAYMENTREQUESTREQUEST_MEMO._serialized_start=3759
  _BTCPAYMENTREQUESTREQUEST_MEMO._serialized_end=3882
  _BTCPAYMENTREQUESTREQUEST_MEMO_TEXTMEMO._serialized_start=3850
  _BTCPAYMENTREQUESTREQUEST_MEMO_TEXTMEMO._serialized_end=3874
  _BTCSIGNMESSAGEREQUEST._serialized_start=3885
  _BTCSIGNMESSAGEREQUEST._serialized_end=4123
  _BTCSIGNMESSAGERESPONSE._serialized_start=4125
  _BTCSIGNMESSAGERESPONSE._serialized_end=4168
  _BTCREQUEST._serialized_start=4171
  _BTCREQUEST._serialized_end=4812
  _BTCRESPONSE._serialized_start=4815
  _BTCRESPONSE._serialized_end=5215
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Shift Cryptosecurity AG
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Abstraction for the transport layer used for transmitting U2F messages."""

from .communication import PhysicalLayer, TransportLayer
from .bitbox_api_protocol import (
    BitBoxNoiseConfig,
    BitBoxCommonAPI,
    Bitbox02Exception,
    UserAbortException,
    AttestationException,
    FirmwareVersionOutdatedException,
    LibraryVersionOutdatedException,
    ERR_DUPLICATE_ENTRY,
    ERR_GENERIC,
    ERR_USER_ABORT,
    HARDENED,
)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2019 Shift Cryptosecurity AG
Copyright 2020 Shift Crypto AG

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from . import antiklepto_pb2
import builtins
import collections.abc
from . import common_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _BTCCoin:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _BTCCoinEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BTCCoin.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    BTC: _BTCCoin.ValueType  # 0
    TBTC: _BTCCoin.ValueType  # 1
    LTC: _BTCCoin.ValueType  # 2
    TLTC: _BTCCoin.ValueType  # 3
    RBTC: _BTCCoin.ValueType  # 4
    """Regtest"""

class BTCCoin(_BTCCoin, metaclass=_BTCCoinEnumTypeWrapper): ...

BTC: BTCCoin.ValueType  # 0
TBTC: BTCCoin.ValueType  # 1
LTC: BTCCoin.ValueType  # 2
TLTC: BTCCoin.ValueType  # 3
RBTC: BTCCoin.ValueType  # 4
"""Regtest"""
global___BTCCoin = BTCCoin

class _BTCOutputType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _BTCOutputTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BTCOutputType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _BTCOutputType.ValueType  # 0
    P2PKH: _BTCOutputType.ValueType  # 1
    P2SH: _BTCOutputType.ValueType  # 2
    P2WPKH: _BTCOutputType.ValueType  # 3
    P2WSH: _BTCOutputType.ValueType  # 4
    P2TR: _BTCOutputType.ValueType  # 5

class BTCOutputType(_BTCOutputType, metaclass=_BTCOutputTypeEnumTypeWrapper): ...

UNKNOWN: BTCOutputType.ValueType  # 0
P2PKH: BTCOutputType.ValueType  # 1
P2SH: BTCOutputType.ValueType  # 2
P2WPKH: BTCOutputType.ValueType  # 3
P2WSH: BTCOutputType.ValueType  # 4
P2TR: BTCOutputType.ValueType  # 5
global___BTCOutputType = BTCOutputType

@typing.final
class BTCScriptConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _SimpleType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _SimpleTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCScriptConfig._SimpleType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        P2WPKH_P2SH: BTCScriptConfig._SimpleType.ValueType  # 0
        P2WPKH: BTCScriptConfig._SimpleType.ValueType  # 1
        P2TR: BTCScriptConfig._SimpleType.ValueType  # 2

    class SimpleType(_SimpleType, metaclass=_SimpleTypeEnumTypeWrapper):
        """SimpleType is a "simple" script: one public key, no additional inputs."""

    P2WPKH_P2SH: BTCScriptConfig.SimpleType.ValueType  # 0
    P2WPKH: BTCScriptConfig.SimpleType.ValueType  # 1
    P2TR: BTCScriptConfig.SimpleType.ValueType  # 2

    @typing.final
    class Multisig(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _ScriptType:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _ScriptTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCScriptConfig.Multisig._ScriptType.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            P2WSH: BTCScriptConfig.Multisig._ScriptType.ValueType  # 0
            """native segwit v0 multisig (bech32 addresses)"""
            P2WSH_P2SH: BTCScriptConfig.Multisig._ScriptType.ValueType  # 1
            """wrapped segwit for legacy address compatibility"""

        class ScriptType(_ScriptType, metaclass=_ScriptTypeEnumTypeWrapper): ...
        P2WSH: BTCScriptConfig.Multisig.ScriptType.ValueType  # 0
        """native segwit v0 multisig (bech32 addresses)"""
        P2WSH_P2SH: BTCScriptConfig.Multisig.ScriptType.ValueType  # 1
        """wrapped segwit for legacy address compatibility"""

        THRESHOLD_FIELD_NUMBER: builtins.int
        XPUBS_FIELD_NUMBER: builtins.int
        OUR_XPUB_INDEX_FIELD_NUMBER: builtins.int
        SCRIPT_TYPE_FIELD_NUMBER: builtins.int
        threshold: builtins.int
        our_xpub_index: builtins.int
        """Index to the xpub of our keystore in xpubs. The keypath to it is provided via
        BTCPubRequest/BTCSignInit.
        """
        script_type: global___BTCScriptConfig.Multisig.ScriptType.ValueType
        @property
        def xpubs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.XPub]:
            """xpubs are acount-level xpubs. Addresses are going to be derived from it using: `m/<change>/<receive>`.
            The number of xpubs defines the number of cosigners.
            """

        def __init__(
            self,
            *,
            threshold: builtins.int = ...,
            xpubs: collections.abc.Iterable[common_pb2.XPub] | None = ...,
            our_xpub_index: builtins.int = ...,
            script_type: global___BTCScriptConfig.Multisig.ScriptType.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["our_xpub_index", b"our_xpub_index", "script_type", b"script_type", "threshold", b"threshold", "xpubs", b"xpubs"]) -> None: ...

    @typing.final
    class Policy(google.protobuf.message.Message):
        """A policy as specified by 'Wallet policies':
        https://github.com/bitcoin/bips/pull/1389
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        POLICY_FIELD_NUMBER: builtins.int
        KEYS_FIELD_NUMBER: builtins.int
        policy: builtins.str
        @property
        def keys(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[common_pb2.KeyOriginInfo]: ...
        def __init__(
            self,
            *,
            policy: builtins.str = ...,
            keys: collections.abc.Iterable[common_pb2.KeyOriginInfo] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["keys", b"keys", "policy", b"policy"]) -> None: ...

    SIMPLE_TYPE_FIELD_NUMBER: builtins.int
    MULTISIG_FIELD_NUMBER: builtins.int
    POLICY_FIELD_NUMBER: builtins.int
    simple_type: global___BTCScriptConfig.SimpleType.ValueType
    @property
    def multisig(self) -> global___BTCScriptConfig.Multisig: ...
    @property
    def policy(self) -> global___BTCScriptConfig.Policy: ...
    def __init__(
        self,
        *,
        simple_type: global___BTCScriptConfig.SimpleType.ValueType = ...,
        multisig: global___BTCScriptConfig.Multisig | None = ...,
        policy: global___BTCScriptConfig.Policy | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["config", b"config", "multisig", b"multisig", "policy", b"policy", "simple_type", b"simple_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["config", b"config", "multisig", b"multisig", "policy", b"policy", "simple_type", b"simple_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["config", b"config"]) -> typing.Literal["simple_type", "multisig", "policy"] | None: ...

global___BTCScriptConfig = BTCScriptConfig

@typing.final
class BTCPubRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _XPubType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _XPubTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCPubRequest._XPubType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        TPUB: BTCPubRequest._XPubType.ValueType  # 0
        XPUB: BTCPubRequest._XPubType.ValueType  # 1
        YPUB: BTCPubRequest._XPubType.ValueType  # 2
        ZPUB: BTCPubRequest._XPubType.ValueType  # 3
        """zpub"""
        VPUB: BTCPubRequest._XPubType.ValueType  # 4
        """vpub"""
        UPUB: BTCPubRequest._XPubType.ValueType  # 5
        CAPITAL_VPUB: BTCPubRequest._XPubType.ValueType  # 6
        """Vpub"""
        CAPITAL_ZPUB: BTCPubRequest._XPubType.ValueType  # 7
        """Zpub"""
        CAPITAL_UPUB: BTCPubRequest._XPubType.ValueType  # 8
        """Upub"""
        CAPITAL_YPUB: BTCPubRequest._XPubType.ValueType  # 9
        """Ypub"""

    class XPubType(_XPubType, metaclass=_XPubTypeEnumTypeWrapper): ...
    TPUB: BTCPubRequest.XPubType.ValueType  # 0
    XPUB: BTCPubRequest.XPubType.ValueType  # 1
    YPUB: BTCPubRequest.XPubType.ValueType  # 2
    ZPUB: BTCPubRequest.XPubType.ValueType  # 3
    """zpub"""
    VPUB: BTCPubRequest.XPubType.ValueType  # 4
    """vpub"""
    UPUB: BTCPubRequest.XPubType.ValueType  # 5
    CAPITAL_VPUB: BTCPubRequest.XPubType.ValueType  # 6
    """Vpub"""
    CAPITAL_ZPUB: BTCPubRequest.XPubType.ValueType  # 7
    """Zpub"""
    CAPITAL_UPUB: BTCPubRequest.XPubType.ValueType  # 8
    """Upub"""
    CAPITAL_YPUB: BTCPubRequest.XPubType.ValueType  # 9
    """Ypub"""

    COIN_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    XPUB_TYPE_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    DISPLAY_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    xpub_type: global___BTCPubRequest.XPubType.ValueType
    display: builtins.bool
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def script_config(self) -> global___BTCScriptConfig: ...
    def __init__(
        self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        keypath: collections.abc.Iterable[builtins.int] | None = ...,
        xpub_type: global___BTCPubRequest.XPubType.ValueType = ...,
        script_config: global___BTCScriptConfig | None = ...,
        display: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["output", b"output", "script_config", b"script_config", "xpub_type", b"xpub_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["coin", b"coin", "display", b"display", "keypath", b"keypath", "output", b"output", "script_config", b"script_config", "xpub_type", b"xpub_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["output", b"output"]) -> typing.Literal["xpub_type", "script_config"] | None: ...

global___BTCPubRequest = BTCPubRequest

@typing.final
class BTCScriptConfigWithKeypath(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    @property
    def script_config(self) -> global___BTCScriptConfig: ...
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        script_config: global___BTCScriptConfig | None = ...,
        keypath: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["script_config", b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["keypath", b"keypath", "script_config", b"script_config"]) -> None: ...

global___BTCScriptConfigWithKeypath = BTCScriptConfigWithKeypath

@typing.final
class BTCSignInitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _FormatUnit:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _FormatUnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCSignInitRequest._FormatUnit.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT: BTCSignInitRequest._FormatUnit.ValueType  # 0
        """According to `coin` (BTC, LTC, etc.)."""
        SAT: BTCSignInitRequest._FormatUnit.ValueType  # 1
        """Only valid for BTC/TBTC, formats as "sat"/"tsat"."""

    class FormatUnit(_FormatUnit, metaclass=_FormatUnitEnumTypeWrapper): ...
    DEFAULT: BTCSignInitRequest.FormatUnit.ValueType  # 0
    """According to `coin` (BTC, LTC, etc.)."""
    SAT: BTCSignInitRequest.FormatUnit.ValueType  # 1
    """Only valid for BTC/TBTC, formats as "sat"/"tsat"."""

    COIN_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIGS_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    NUM_INPUTS_FIELD_NUMBER: builtins.int
    NUM_OUTPUTS_FIELD_NUMBER: builtins.int
    LOCKTIME_FIELD_NUMBER: builtins.int
    FORMAT_UNIT_FIELD_NUMBER: builtins.int
    CONTAINS_SILENT_PAYMENT_OUTPUTS_FIELD_NUMBER: builtins.int
    OUTPUT_SCRIPT_CONFIGS_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    version: builtins.int
    """must be 1 or 2"""
    num_inputs: builtins.int
    num_outputs: builtins.int
    locktime: builtins.int
    """must be <500000000"""
    format_unit: global___BTCSignInitRequest.FormatUnit.ValueType
    contains_silent_payment_outputs: builtins.bool
    @property
    def script_configs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BTCScriptConfigWithKeypath]:
        """used script configs in inputs and changes"""

    @property
    def output_script_configs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BTCScriptConfigWithKeypath]:
        """used script configs for outputs that send to an address of the same keystore, but not
        necessarily the same account (as defined by `script_configs` above).
        """

    def __init__(
        self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        script_configs: collections.abc.Iterable[global___BTCScriptConfigWithKeypath] | None = ...,
        version: builtins.int = ...,
        num_inputs: builtins.int = ...,
        num_outputs: builtins.int = ...,
        locktime: builtins.int = ...,
        format_unit: global___BTCSignInitRequest.FormatUnit.ValueType = ...,
        contains_silent_payment_outputs: builtins.bool = ...,
        output_script_configs: collections.abc.Iterable[global___BTCScriptConfigWithKeypath] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["coin", b"coin", "contains_silent_payment_outputs", b"contains_silent_payment_outputs", "format_unit", b"format_unit", "locktime", b"locktime", "num_inputs", b"num_inputs", "num_outputs", b"num_outputs", "output_script_configs", b"output_script_configs", "script_configs", b"script_configs", "version", b"version"]) -> None: ...

global___BTCSignInitRequest = BTCSignInitRequest

@typing.final
class BTCSignNextResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCSignNextResponse._Type.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INPUT: BTCSignNextResponse._Type.ValueType  # 0
        OUTPUT: BTCSignNextResponse._Type.ValueType  # 1
        DONE: BTCSignNextResponse._Type.ValueType  # 2
        PREVTX_INIT: BTCSignNextResponse._Type.ValueType  # 3
        """For the previous transaction at input `index`."""
        PREVTX_INPUT: BTCSignNextResponse._Type.ValueType  # 4
        PREVTX_OUTPUT: BTCSignNextResponse._Type.ValueType  # 5
        HOST_NONCE: BTCSignNextResponse._Type.ValueType  # 6
        PAYMENT_REQUEST: BTCSignNextResponse._Type.ValueType  # 7

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    INPUT: BTCSignNextResponse.Type.ValueType  # 0
    OUTPUT: BTCSignNextResponse.Type.ValueType  # 1
    DONE: BTCSignNextResponse.Type.ValueType  # 2
    PREVTX_INIT: BTCSignNextResponse.Type.ValueType  # 3
    """For the previous transaction at input `index`."""
    PREVTX_INPUT: BTCSignNextResponse.Type.ValueType  # 4
    PREVTX_OUTPUT: BTCSignNextResponse.Type.ValueType  # 5
    HOST_NONCE: BTCSignNextResponse.Type.ValueType  # 6
    PAYMENT_REQUEST: BTCSignNextResponse.Type.ValueType  # 7

    TYPE_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    HAS_SIGNATURE_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    PREV_INDEX_FIELD_NUMBER: builtins.int
    ANTI_KLEPTO_SIGNER_COMMITMENT_FIELD_NUMBER: builtins.int
    GENERATED_OUTPUT_PKSCRIPT_FIELD_NUMBER: builtins.int
    SILENT_PAYMENT_DLEQ_PROOF_FIELD_NUMBER: builtins.int
    type: global___BTCSignNextResponse.Type.ValueType
    index: builtins.int
    """index of the current input or output"""
    has_signature: builtins.bool
    """only as a response to BTCSignInputRequest"""
    signature: builtins.bytes
    """64 bytes (32 bytes big endian R, 32 bytes big endian S). Only if has_signature is true."""
    prev_index: builtins.int
    """Previous tx's input/output index in case of PREV_INPUT or PREV_OUTPUT, for the input at `index`."""
    generated_output_pkscript: builtins.bytes
    """Generated output. The host *must* verify its correctness using `silent_payment_dleq_proof`."""
    silent_payment_dleq_proof: builtins.bytes
    @property
    def anti_klepto_signer_commitment(self) -> antiklepto_pb2.AntiKleptoSignerCommitment: ...
    def __init__(
        self,
        *,
        type: global___BTCSignNextResponse.Type.ValueType = ...,
        index: builtins.int = ...,
        has_signature: builtins.bool = ...,
        signature: builtins.bytes = ...,
        prev_index: builtins.int = ...,
        anti_klepto_signer_commitment: antiklepto_pb2.AntiKleptoSignerCommitment | None = ...,
        generated_output_pkscript: builtins.bytes = ...,
        silent_payment_dleq_proof: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["anti_klepto_signer_commitment", b"anti_klepto_signer_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["anti_klepto_signer_commitment", b"anti_klepto_signer_commitment", "generated_output_pkscript", b"generated_output_pkscript", "has_signature", b"has_signature", "index", b"index", "prev_index", b"prev_index", "signature", b"signature", "silent_payment_dleq_proof", b"silent_payment_dleq_proof", "type", b"type"]) -> None: ...

global___BTCSignNextResponse = BTCSignNextResponse

@typing.final
class BTCSignInputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PREVOUTHASH_FIELD_NUMBER: builtins.int
    PREVOUTINDEX_FIELD_NUMBER: builtins.int
    PREVOUTVALUE_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_INDEX_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    prevOutHash: builtins.bytes
    prevOutIndex: builtins.int
    prevOutValue: builtins.int
    sequence: builtins.int
    """must be 0xffffffff-2, 0xffffffff-1 or 0xffffffff"""
    script_config_index: builtins.int
    """References a script config from BTCSignInitRequest"""
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """all inputs must be ours."""

    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    def __init__(
        self,
        *,
        prevOutHash: builtins.bytes = ...,
        prevOutIndex: builtins.int = ...,
        prevOutValue: builtins.int = ...,
        sequence: builtins.int = ...,
        keypath: collections.abc.Iterable[builtins.int] | None = ...,
        script_config_index: builtins.int = ...,
        host_nonce_commitment: antiklepto_pb2.AntiKleptoHostNonceCommitment | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["host_nonce_commitment", b"host_nonce_commitment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["host_nonce_commitment", b"host_nonce_commitment", "keypath", b"keypath", "prevOutHash", b"prevOutHash", "prevOutIndex", b"prevOutIndex", "prevOutValue", b"prevOutValue", "script_config_index", b"script_config_index", "sequence", b"sequence"]) -> None: ...

global___BTCSignInputRequest = BTCSignInputRequest

@typing.final
class BTCSignOutputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class SilentPayment(google.protobuf.message.Message):
        """https://github.com/bitcoin/bips/blob/master/bip-0352.mediawiki"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ADDRESS_FIELD_NUMBER: builtins.int
        address: builtins.str
        def __init__(
            self,
            *,
            address: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["address", b"address"]) -> None: ...

    OURS_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    PAYLOAD_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_INDEX_FIELD_NUMBER: builtins.int
    PAYMENT_REQUEST_INDEX_FIELD_NUMBER: builtins.int
    SILENT_PAYMENT_FIELD_NUMBER: builtins.int
    OUTPUT_SCRIPT_CONFIG_INDEX_FIELD_NUMBER: builtins.int
    ours: builtins.bool
    type: global___BTCOutputType.ValueType
    """if ours is false"""
    value: builtins.int
    """20 bytes for p2pkh, p2sh, pw2wpkh. 32 bytes for p2wsh."""
    payload: builtins.bytes
    """if ours is false. Renamed from `hash`."""
    script_config_index: builtins.int
    """If ours is true and `output_script_config_index` is absent. References a script config from
    BTCSignInitRequest. This allows change output identification and allows us to identify
    non-change outputs to the same account, so we can display this info to the user.
    """
    payment_request_index: builtins.int
    output_script_config_index: builtins.int
    """If ours is true. If set, `script_config_index` is ignored. References an output script config
    from BTCSignInitRequest. This enables verification that an output belongs to the same keystore,
    even if it is from a different account than we spend from, allowing us to display this info to
    the user.
    """
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """if ours is true"""

    @property
    def silent_payment(self) -> global___BTCSignOutputRequest.SilentPayment:
        """If provided, `type` and `payload` is ignored. The generated output pkScript is returned in
        BTCSignNextResponse. `contains_silent_payment_outputs` in the init request must be true.
        """

    def __init__(
        self,
        *,
        ours: builtins.bool = ...,
        type: global___BTCOutputType.ValueType = ...,
        value: builtins.int = ...,
        payload: builtins.bytes = ...,
        keypath: collections.abc.Iterable[builtins.int] | None = ...,
        script_config_index: builtins.int = ...,
        payment_request_index: builtins.int | None = ...,
        silent_payment: global___BTCSignOutputRequest.SilentPayment | None = ...,
        output_script_config_index: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_output_script_config_index", b"_output_script_config_index", "_payment_request_index", b"_payment_request_index", "output_script_config_index", b"output_script_config_index", "payment_request_index", b"payment_request_index", "silent_payment", b"silent_payment"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_output_script_config_index", b"_output_script_config_index", "_payment_request_index", b"_payment_request_index", "keypath", b"keypath", "ours", b"ours", "output_script_config_index", b"output_script_config_index", "payload", b"payload", "payment_request_index", b"payment_request_index", "script_config_index", b"script_config_index", "silent_payment", b"silent_payment", "type", b"type", "value", b"value"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_output_script_config_index", b"_output_script_config_index"]) -> typing.Literal["output_script_config_index"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_payment_request_index", b"_payment_request_index"]) -> typing.Literal["payment_request_index"] | None: ...

global___BTCSignOutputRequest = BTCSignOutputRequest

@typing.final
class BTCScriptConfigRegistration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COIN_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    KEYPATH_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    @property
    def script_config(self) -> global___BTCScriptConfig: ...
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Unused for policy registrations."""

    def __init__(
        self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        script_config: global___BTCScriptConfig | None = ...,
        keypath: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["script_config", b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["coin", b"coin", "keypath", b"keypath", "script_config", b"script_config"]) -> None: ...

global___BTCScriptConfigRegistration = BTCScriptConfigRegistration

@typing.final
class BTCSuccess(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___BTCSuccess = BTCSuccess

@typing.final
class BTCIsScriptConfigRegisteredRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REGISTRATION_FIELD_NUMBER: builtins.int
    @property
    def registration(self) -> global___BTCScriptConfigRegistration: ...
    def __init__(
        self,
        *,
        registration: global___BTCScriptConfigRegistration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["registration", b"registration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["registration", b"registration"]) -> None: ...

global___BTCIsScriptConfigRegisteredRequest = BTCIsScriptConfigRegisteredRequest

@typing.final
class BTCIsScriptConfigRegisteredResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IS_REGISTERED_FIELD_NUMBER: builtins.int
    is_registered: builtins.bool
    def __init__(
        self,
        *,
        is_registered: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["is_registered", b"is_registered"]) -> None: ...

global___BTCIsScriptConfigRegisteredResponse = BTCIsScriptConfigRegisteredResponse

@typing.final
class BTCRegisterScriptConfigRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _XPubType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _XPubTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[BTCRegisterScriptConfigRequest._XPubType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        AUTO_ELECTRUM: BTCRegisterScriptConfigRequest._XPubType.ValueType  # 0
        """Automatically choose to match Electrum's xpub format (e.g. Zpub/Vpub for p2wsh multisig mainnet/testnet)."""
        AUTO_XPUB_TPUB: BTCRegisterScriptConfigRequest._XPubType.ValueType  # 1
        """Always xpub for mainnets, tpub for testnets."""

    class XPubType(_XPubType, metaclass=_XPubTypeEnumTypeWrapper): ...
    AUTO_ELECTRUM: BTCRegisterScriptConfigRequest.XPubType.ValueType  # 0
    """Automatically choose to match Electrum's xpub format (e.g. Zpub/Vpub for p2wsh multisig mainnet/testnet)."""
    AUTO_XPUB_TPUB: BTCRegisterScriptConfigRequest.XPubType.ValueType  # 1
    """Always xpub for mainnets, tpub for testnets."""

    REGISTRATION_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    XPUB_TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    """If empty, the name is entered on the device instead."""
    xpub_type: global___BTCRegisterScriptConfigRequest.XPubType.ValueType
    @property
    def registration(self) -> global___BTCScriptConfigRegistration: ...
    def __init__(
        self,
        *,
        registration: global___BTCScriptConfigRegistration | None = ...,
        name: builtins.str = ...,
        xpub_type: global___BTCRegisterScriptConfigRequest.XPubType.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["registration", b"registration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "registration", b"registration", "xpub_type", b"xpub_type"]) -> None: ...

global___BTCRegisterScriptConfigRequest = BTCRegisterScriptConfigRequest

@typing.final
class BTCPrevTxInitRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    NUM_INPUTS_FIELD_NUMBER: builtins.int
    NUM_OUTPUTS_FIELD_NUMBER: builtins.int
    LOCKTIME_FIELD_NUMBER: builtins.int
    version: builtins.int
    num_inputs: builtins.int
    num_outputs: builtins.int
    locktime: builtins.int
    def __init__(
        self,
        *,
        version: builtins.int = ...,
        num_inputs: builtins.int = ...,
        num_outputs: builtins.int = ...,
        locktime: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["locktime", b"locktime", "num_inputs", b"num_inputs", "num_outputs", b"num_outputs", "version", b"version"]) -> None: ...

global___BTCPrevTxInitRequest = BTCPrevTxInitRequest

@typing.final
class BTCPrevTxInputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PREV_OUT_HASH_FIELD_NUMBER: builtins.int
    PREV_OUT_INDEX_FIELD_NUMBER: builtins.int
    SIGNATURE_SCRIPT_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    prev_out_hash: builtins.bytes
    prev_out_index: builtins.int
    signature_script: builtins.bytes
    sequence: builtins.int
    def __init__(
        self,
        *,
        prev_out_hash: builtins.bytes = ...,
        prev_out_index: builtins.int = ...,
        signature_script: builtins.bytes = ...,
        sequence: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["prev_out_hash", b"prev_out_hash", "prev_out_index", b"prev_out_index", "sequence", b"sequence", "signature_script", b"signature_script"]) -> None: ...

global___BTCPrevTxInputRequest = BTCPrevTxInputRequest

@typing.final
class BTCPrevTxOutputRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    PUBKEY_SCRIPT_FIELD_NUMBER: builtins.int
    value: builtins.int
    pubkey_script: builtins.bytes
    def __init__(
        self,
        *,
        value: builtins.int = ...,
        pubkey_script: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["pubkey_script", b"pubkey_script", "value", b"value"]) -> None: ...

global___BTCPrevTxOutputRequest = BTCPrevTxOutputRequest

@typing.final
class BTCPaymentRequestRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Memo(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class TextMemo(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            NOTE_FIELD_NUMBER: builtins.int
            note: builtins.str
            def __init__(
                self,
                *,
                note: builtins.str = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing.Literal["note", b"note"]) -> None: ...

        TEXT_MEMO_FIELD_NUMBER: builtins.int
        @property
        def text_memo(self) -> global___BTCPaymentRequestRequest.Memo.TextMemo: ...
        def __init__(
            self,
            *,
            text_memo: global___BTCPaymentRequestRequest.Memo.TextMemo | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["memo", b"memo", "text_memo", b"text_memo"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["memo", b"memo", "text_memo", b"text_memo"]) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["memo", b"memo"]) -> typing.Literal["text_memo"] | None: ...

    RECIPIENT_NAME_FIELD_NUMBER: builtins.int
    MEMOS_FIELD_NUMBER: builtins.int
    NONCE_FIELD_NUMBER: builtins.int
    TOTAL_AMOUNT_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    recipient_name: builtins.str
    nonce: builtins.bytes
    total_amount: builtins.int
    signature: builtins.bytes
    @property
    def memos(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___BTCPaymentRequestRequest.Memo]: ...
    def __init__(
        self,
        *,
        recipient_name: builtins.str = ...,
        memos: collections.abc.Iterable[global___BTCPaymentRequestRequest.Memo] | None = ...,
        nonce: builtins.bytes = ...,
        total_amount: builtins.int = ...,
        signature: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["memos", b"memos", "nonce", b"nonce", "recipient_name", b"recipient_name", "signature", b"signature", "total_amount", b"total_amount"]) -> None: ...

global___BTCPaymentRequestRequest = BTCPaymentRequestRequest

@typing.final
class BTCSignMessageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COIN_FIELD_NUMBER: builtins.int
    SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    MSG_FIELD_NUMBER: builtins.int
    HOST_NONCE_COMMITMENT_FIELD_NUMBER: builtins.int
    coin: global___BTCCoin.ValueType
    msg: builtins.bytes
    @property
    def script_config(self) -> global___BTCScriptConfigWithKeypath: ...
    @property
    def host_nonce_commitment(self) -> antiklepto_pb2.AntiKleptoHostNonceCommitment: ...
    def __init__(
        self,
        *,
        coin: global___BTCCoin.ValueType = ...,
        script_config: global___BTCScriptConfigWithKeypath | None = ...,
        msg: builtins.bytes = ...,
        host_nonce_commitment: antiklepto_pb2.AntiKleptoHostNonceCommitment | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["host_nonce_commitment", b"host_nonce_commitment", "script_config", b"script_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["coin", b"coin", "host_nonce_commitment", b"host_nonce_commitment", "msg", b"msg", "script_config", b"script_config"]) -> None: ...

global___BTCSignMessageRequest = BTCSignMessageRequest

@typing.final
class BTCSignMessageResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SIGNATURE_FIELD_NUMBER: builtins.int
    signature: builtins.bytes
    """65 bytes (32 bytes big endian R, 32 bytes big endian S, 1 recid)."""
    def __init__(
        self,
        *,
        signature: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["signature", b"signature"]) -> None: ...

global___BTCSignMessageResponse = BTCSignMessageResponse

@typing.final
class BTCRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IS_SCRIPT_CONFIG_REGISTERED_FIELD_NUMBER: builtins.int
    REGISTER_SCRIPT_CONFIG_FIELD_NUMBER: builtins.int
    PREVTX_INIT_FIELD_NUMBER: builtins.int
    PREVTX_INPUT_FIELD_NUMBER: builtins.int
    PREVTX_OUTPUT_FIELD_NUMBER: builtins.int
    SIGN_MESSAGE_FIELD_NUMBER: builtins.int
    ANTIKLEPTO_SIGNATURE_FIELD_NUMBER: builtins.int
    PAYMENT_REQUEST_FIELD_NUMBER: builtins.int
    @property
    def is_script_config_registered(self) -> global___BTCIsScriptConfigRegisteredRequest: ...
    @property
    def register_script_config(self) -> global___BTCRegisterScriptConfigRequest: ...
    @property
    def prevtx_init(self) -> global___BTCPrevTxInitRequest: ...
    @property
    def prevtx_input(self) -> global___BTCPrevTxInputRequest: ...
    @property
    def prevtx_output(self) -> global___BTCPrevTxOutputRequest: ...
    @property
    def sign_message(self) -> global___BTCSignMessageRequest: ...
    @property
    def antiklepto_signature(self) -> antiklepto_pb2.AntiKleptoSignatureRequest: ...
    @property
    def payment_request(self) -> global___BTCPaymentRequestRequest: ...
    def __init__(
        self,
        *,
        is_script_config_registered: global___BTCIsScriptConfigRegisteredRequest | None = ...,
        register_script_config: global___BTCRegisterScriptConfigRequest | None = ...,
        prevtx_init: global___BTCPrevTxInitRequest | None = ...,
        prevtx_input: global___BTCPrevTxInputRequest | None = ...,
        prevtx_output: global___BTCPrevTxOutputRequest | None = ...,
        sign_message: global___BTCSignMessageRequest | None = ...,
        antiklepto_signature: antiklepto_pb2.AntiKleptoSignatureRequest | None = ...,
        payment_request: global___BTCPaymentRequestRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["antiklepto_signature", b"antiklepto_signature", "is_script_config_registered", b"is_script_config_registered", "payment_request", b"payment_request", "prevtx_init", b"prevtx_init", "prevtx_input", b"prevtx_input", "prevtx_output", b"prevtx_output", "register_script_config", b"register_script_config", "request", b"request", "sign_message", b"sign_message"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["antiklepto_signature", b"antiklepto_signature", "is_script_config_registered", b"is_script_config_registered", "payment_request", b"payment_request", "prevtx_init", b"prevtx_init", "prevtx_input", b"prevtx_input", "prevtx_output", b"prevtx_output", "register_script_config", b"register_script_config", "request", b"request", "sign_message", b"sign_message"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["request", b"request"]) -> typing.Literal["is_script_config_registered", "register_script_config", "prevtx_init", "prevtx_input", "prevtx_output", "sign_message", "antiklepto_signature", "payment_request"] | None: ...

global___BTCRequest = BTCRequest

@typing.final
class BTCResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    IS_SCRIPT_CONFIG_REGISTERED_FIELD_NUMBER: builtins.int
    SIGN_NEXT_FIELD_NUMBER: builtins.int
    SIGN_MESSAGE_FIELD_NUMBER: builtins.int
    ANTIKLEPTO_SIGNER_COMMITMENT_FIELD_NUMBER: builtins.int
    @property
    def success(self) -> global___BTCSuccess: ...
    @property
    def is_script_config_registered(self) -> global___BTCIsScriptConfigRegisteredResponse: ...
    @property
    def sign_next(self) -> global___BTCSignNextResponse: ...
    @property
    def sign_message(self) -> global___BTCSignMessageResponse: ...
    @property
    def antiklepto_signer_commitment(self) -> antiklepto_pb2.AntiKleptoSignerCommitment: ...
    def __init__(
        self,
        *,
        success: global___BTCSuccess | None = ...,
        is_script_config_registered: global___BTCIsScriptConfigRegisteredResponse | None = ...,
        sign_next: global___BTCSignNextResponse | None = ...,
        sign_message: global___BTCSignMessageResponse | None = ...,
        antiklepto_signer_commitment: antiklepto_pb2.AntiKleptoSignerCommitment | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["antiklepto_signer_commitment", b"antiklepto_signer_commitment", "is_script_config_registered", b"is_script_config_registered", "response", b"response", "sign_message", b"sign_message", "sign_next", b"sign_next", "success", b"success"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["antiklepto_signer_commitment", b"antiklepto_signer_commitment", "is_script_config_registered", b"is_script_config_registered", "response", b"response", "sign_message", b"sign_message", "sign_next", b"sign_next", "success", b"success"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["response", b"response"]) -> typing.Literal["success", "is_script_config_registered", "sign_next", "sign_message", "antiklepto_signer_commitment"] | None: ...

global___BTCResponse = BTCResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
This function can be used to get an identifying xpub at the keypath m/4541509'/1112098098'"
The keypath argument has to be m/4541509'/1112098098'
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.empty_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ElectrumEncryptionKeyRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEYPATH_FIELD_NUMBER: builtins.int
    @property
    def keypath(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        keypath: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["keypath", b"keypath"]) -> None: ...

global___ElectrumEncryptionKeyRequest = ElectrumEncryptionKeyRequest

@typing.final
class ElectrumEncryptionKeyResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["key", b"key"]) -> None: ...

global___ElectrumEncryptionKeyResponse = ElectrumEncryptionKeyResponse

@typing.final
class BIP85Request(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AppLn(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ACCOUNT_NUMBER_FIELD_NUMBER: builtins.int
        account_number: builtins.int
        def __init__(
            self,
            *,
            account_number: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["account_number", b"account_number"]) -> None: ...

    BIP39_FIELD_NUMBER: builtins.int
    LN_FIELD_NUMBER: builtins.int
    @property
    def bip39(self) -> google.protobuf.empty_pb2.Empty: ...
    @property
    def ln(self) -> global___BIP85Request.AppLn: ...
    def __init__(
        self,
        *,
        bip39: google.protobuf.empty_pb2.Empty | None = ...,
        ln: global___BIP85Request.AppLn | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["app", b"app", "bip39", b"bip39", "ln", b"ln"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["app", b"app", "bip39", b"bip39", "ln", b"ln"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["app", b"app"]) -> typing.Literal["bip39", "ln"] | None: ...

global___BIP85Request = BIP85Request

@typing.final
class BIP85Response(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BIP39_FIELD_NUMBER: builtins.int
    LN_FIELD_NUMBER: builtins.int
    ln: builtins.bytes
    @property
    def bip39(self) -> google.protobuf.empty_pb2.Empty: ...
    def __init__(
        self,
        *,
        bip39: google.protobuf.empty_pb2.Empty | None = ...,
        ln: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["app", b"app", "bip39", b"bip39", "ln", b"ln"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["app", b"app", "bip39", b"bip39", "ln", b"ln"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["app", b"app"]) -> typing.Literal["bip39", "ln"] | None: ...

global___BIP85Response = BIP85Response



"""
Keepkey
*******
"""

from ..common import Chain
from ..errors import (
    DEVICE_NOT_INITIALIZED,
    DeviceNotReadyError,
    common_err_msgs,
    handle_errors,
)
from .trezorlib import protobuf
from .trezorlib.transport import (
    hid,
    udp,
    webusb,
)
from .trezor import TrezorClient, HID_IDS, WEBUSB_IDS
from .trezorlib.mapping import DEFAULT_MAPPING
from .trezorlib.messages import (
    DebugLinkState,
    Features,
    ResetDevice,
)
from .trezorlib.models import TrezorModel

from typing import (
    Any,
    Dict,
    List,
    Optional,
)

py_enumerate = enumerate # Need to use the enumerate built-in but there's another function already named that

KEEPKEY_HID_IDS = {(0x2B24, 0x0001)}
KEEPKEY_WEBUSB_IDS = {(0x2B24, 0x0002)}
KEEPKEY_SIMULATOR_PATH = '127.0.0.1:11044'

HID_IDS.update(KEEPKEY_HID_IDS)
WEBUSB_IDS.update(KEEPKEY_WEBUSB_IDS)


class KeepkeyFeatures(Features): # type: ignore
    MESSAGE_WIRE_TYPE = 17
    FIELDS = {
        1: protobuf.Field("vendor", "string", repeated=False, required=False),
        2: protobuf.Field("major_version", "uint32", repeated=False, required=True),
        3: protobuf.Field("minor_version", "uint32", repeated=False, required=True),
        4: protobuf.Field("patch_version", "uint32", repeated=False, required=True),
        5: protobuf.Field("bootloader_mode", "bool", repeated=False, required=False),
        6: protobuf.Field("device_id", "string", repeated=False, required=False),
        7: protobuf.Field("pin_protection", "bool", repeated=False, required=False),
        8: protobuf.Field("passphrase_protection", "bool", repeated=False, required=False),
        9: protobuf.Field("language", "string", repeated=False, required=False),
        10: protobuf.Field("label", "string", repeated=False, required=False),
        12: protobuf.Field("initialized", "bool", repeated=False, required=False),
        13: protobuf.Field("revision", "bytes", repeated=False, required=False),
        14: protobuf.Field("bootloader_hash", "bytes", repeated=False, required=False),
        15: protobuf.Field("imported", "bool", repeated=False, required=False),
        16: protobuf.Field("unlocked", "bool", repeated=False, required=False),
        17: protobuf.Field("passphrase_cached", "bool", repeated=False, required=False),
        21: protobuf.Field("model", "string", repeated=False, required=False),
        22: protobuf.Field("firmware_variant", "string", repeated=False, required=False),
        23: protobuf.Field("firmware_hash", "bytes", repeated=False, required=False),
        24: protobuf.Field("no_backup", "bool", repeated=False, required=False),
    }

    def __init__(
        self,
        *,
        firmware_variant: Optional[str] = None,
        firmware_hash: Optional[bytes] = None,
        passphrase_cached: Optional[bool] = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)
        self.passphrase_cached = passphrase_cached
        self.firmware_variant = firmware_variant
        self.firmware_hash = firmware_hash


class KeepkeyResetDevice(ResetDevice): # type: ignore
    MESSAGE_WIRE_TYPE = 14
    FIELDS = {
        1: protobuf.Field("display_random", "bool", repeated=False, required=False),
        2: protobuf.Field("strength", "uint32", repeated=False, required=False),
        3: protobuf.Field("passphrase_protection", "bool", repeated=False, required=False),
        4: protobuf.Field("pin_protection", "bool", repeated=False, required=False),
        5: protobuf.Field("language", "string", repeated=False, required=False),
        6: protobuf.Field("label", "string", repeated=False, required=False),
        7: protobuf.Field("no_backup", "bool", repeated=False, required=False),
        8: protobuf.Field("auto_lock_delay_ms", "uint32", repeated=False, required=False),
        9: protobuf.Field("u2f_counter", "uint32", repeated=False, required=False),
    }

    def __init__(
        self,
        *,
        auto_lock_delay_ms: Optional[int] = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)
        self.auto_lock_delay_ms = auto_lock_delay_ms


class KeepkeyDebugLinkState(DebugLinkState): # type: ignore
    MESSAGE_WIRE_TYPE = 102
    FIELDS = {
        1: protobuf.Field("layout", "bytes", repeated=False, required=False),
        2: protobuf.Field("pin", "string", repeated=False, required=False),
        3: protobuf.Field("matrix", "string", repeated=False, required=False),
        4: protobuf.Field("mnemonic_secret", "bytes", repeated=False, required=False),
        5: protobuf.Field("node", "HDNodeType", repeated=False, required=False),
        6: protobuf.Field("passphrase_protection", "bool", repeated=False, required=False),
        7: protobuf.Field("reset_word", "string", repeated=False, required=False),
        8: protobuf.Field("reset_entropy", "bytes", repeated=False, required=False),
        9: protobuf.Field("recovery_fake_word", "string", repeated=False, required=False),
        10: protobuf.Field("recovery_word_pos", "uint32", repeated=False, required=False),
        11: protobuf.Field("recovery_cipher", "string", repeated=False, required=False),
        12: protobuf.Field("recovery_auto_completed_word", "string", repeated=False, required=False),
        13: protobuf.Field("firmware_hash", "bytes", repeated=False, required=False),
        14: protobuf.Field("storage_hash", "bytes", repeated=False, required=False),
    }

    def __init__(
        self,
        *,
        recovery_cipher: Optional[str] = None,
        recovery_auto_completed_word: Optional[str] = None,
        firmware_hash: Optional[bytes] = None,
        storage_hash: Optional[bytes] = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)
        self.recovery_cipher = recovery_cipher
        self.recovery_auto_completed_word = recovery_auto_completed_word
        self.firmware_hash = firmware_hash
        self.storage_hash = storage_hash


class KeepkeyClient(TrezorClient):
    def __init__(self, path: str, password: Optional[str] = None, expert: bool = False, chain: Chain = Chain.MAIN) -> None:
        """
        The `KeepkeyClient` is a `HardwareWalletClient` for interacting with the Keepkey.

        As Keepkeys are clones of the Trezor 1, please refer to `TrezorClient` for documentation.
        """
        model = TrezorModel(
            name="K1-14M",
            internal_name="keepkey",
            minimum_version=(0, 0, 0),
            vendors=("keepkey.com"),
            usb_ids=(), # unused
            default_mapping=DEFAULT_MAPPING,
        )
        model.default_mapping.register(KeepkeyFeatures)
        model.default_mapping.register(KeepkeyResetDevice)
        if path.startswith("udp"):
            model.default_mapping.register(KeepkeyDebugLinkState)

        super(KeepkeyClient, self).__init__(path, password, expert, chain, KEEPKEY_HID_IDS, KEEPKEY_WEBUSB_IDS, KEEPKEY_SIMULATOR_PATH, model)
        self.type = 'Keepkey'

    def can_sign_taproot(self) -> bool:
        """
        The KeepKey does not support Taproot yet.

        :returns: False, always
        """
        return False


def enumerate(password: Optional[str] = None, expert: bool = False, chain: Chain = Chain.MAIN, allow_emulators: bool = False) -> List[Dict[str, Any]]:
    results = []
    devs = hid.HidTransport.enumerate(usb_ids=KEEPKEY_HID_IDS)
    devs.extend(webusb.WebUsbTransport.enumerate(usb_ids=KEEPKEY_WEBUSB_IDS))
    if allow_emulators:
        devs.extend(udp.UdpTransport.enumerate(KEEPKEY_SIMULATOR_PATH))
    for dev in devs:
        d_data: Dict[str, Any] = {}

        d_data['type'] = 'keepkey'
        d_data['model'] = 'keepkey'
        d_data['path'] = dev.get_path()

        client = None

        with handle_errors(common_err_msgs["enumerate"], d_data):
            client = KeepkeyClient(d_data['path'], password)
            try:
                client.client.refresh_features()
            except TypeError:
                continue
            if 'keepkey' not in client.client.features.vendor:
                continue

            d_data['label'] = client.client.features.label
            if d_data['path'].startswith('udp:'):
                d_data['model'] += '_simulator'

            d_data['needs_pin_sent'] = client.client.features.pin_protection and not client.client.features.unlocked
            d_data['needs_passphrase_sent'] = client.client.features.passphrase_protection # always need the passphrase sent for Keepkey if it has passphrase protection enabled
            if d_data['needs_pin_sent']:
                raise DeviceNotReadyError('Keepkey is locked. Unlock by using \'promptpin\' and then \'sendpin\'.')
            if d_data['needs_passphrase_sent'] and password is None:
                raise DeviceNotReadyError("Passphrase needs to be specified before the fingerprint information can be retrieved")
            if client.client.features.initialized:
                d_data['fingerprint'] = client.get_master_fingerprint().hex()
                d_data['needs_passphrase_sent'] = False # Passphrase is always needed for the above to have worked, so it's already sent
            else:
                d_data['error'] = 'Not initialized'
                d_data['code'] = DEVICE_NOT_INITIALIZED

        if client:
            client.close()

        results.append(d_data)
    return results



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63ommon.proto\x12\x14shiftcrypto.bitbox02\"\x1a\n\x0bPubResponse\x12\x0b\n\x03pub\x18\x01 \x01(\t\"\x18\n\x16RootFingerprintRequest\".\n\x17RootFingerprintResponse\x12\x13\n\x0b\x66ingerprint\x18\x01 \x01(\x0c\"l\n\x04XPub\x12\r\n\x05\x64\x65pth\x18\x01 \x01(\x0c\x12\x1a\n\x12parent_fingerprint\x18\x02 \x01(\x0c\x12\x11\n\tchild_num\x18\x03 \x01(\r\x12\x12\n\nchain_code\x18\x04 \x01(\x0c\x12\x12\n\npublic_key\x18\x05 \x01(\x0c\"\x1a\n\x07Keypath\x12\x0f\n\x07keypath\x18\x01 \x03(\r\"d\n\rKeyOriginInfo\x12\x18\n\x10root_fingerprint\x18\x01 \x01(\x0c\x12\x0f\n\x07keypath\x18\x02 \x03(\r\x12(\n\x04xpub\x18\x03 \x01(\x0b\x32\x1a.shiftcrypto.bitbox02.XPubb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PUBRESPONSE._serialized_start=38
  _PUBRESPONSE._serialized_end=64
  _ROOTFINGERPRINTREQUEST._serialized_start=66
  _ROOTFINGERPRINTREQUEST._serialized_end=90
  _ROOTFINGERPRINTRESPONSE._serialized_start=92
  _ROOTFINGERPRINTRESPONSE._serialized_end=138
  _XPUB._serialized_start=140
  _XPUB._serialized_end=248
  _KEYPATH._serialized_start=250
  _KEYPATH._serialized_end=276
  _KEYORIGININFO._serialized_start=278
  _KEYORIGININFO._serialized_end=378
# @@protoc_insertion_point(module_scope)



"""ledgercomm.interfaces.tcp_client module."""

import socket
from typing import Tuple

from .comm import Comm
from ..log import LOG


class TCPClient(Comm):
    """TCPClient class.

    Mainly used to connect to the TCP server of the Speculos emulator.

    Parameters
    ----------
    server : str
        IP address of the TCP server.
    port : int
        Port of the TCP server.

    Attributes
    ----------
    server : str
        IP address of the TCP server.
    port : int
        Port of the TCP server.
    socket : socket.socket
        TCP socket to communicate with the server.
    __opened : bool
        Whether the TCP socket is opened or not.

    """

    def __init__(self, server: str, port: int) -> None:
        """Init constructor of TCPClient."""
        self.server: str = server
        self.port: int = port
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__opened: bool = False

    def open(self) -> None:
        """Open connection to TCP socket with `self.server` and `self.port`.

        Returns
        -------
        None

        """
        if not self.__opened:
            self.socket.connect((self.server, self.port))
            self.__opened = True

    def send(self, data: bytes) -> int:
        """Send `data` through TCP socket `self.socket`.

        Parameters
        ----------
        data : bytes
            Bytes of data to send.

        Returns
        -------
        int
            Total lenght of data sent through TCP socket.

        """
        if not data:
            raise Exception("Can't send empty data!")

        LOG.debug("=> %s", data.hex())
        data_len: bytes = int.to_bytes(len(data), 4, byteorder="big")

        return self.socket.send(data_len + data)

    def recv(self) -> Tuple[int, bytes]:
        """Receive data through TCP socket `self.socket`.

        Blocking IO.

        Returns
        -------
        Tuple[int, bytes]
            A pair (sw, rdata) containing the status word and response data.

        """
        length: int = int.from_bytes(self.socket.recv(4), byteorder="big")
        rdata: bytes = self.socket.recv(length)
        sw: int = int.from_bytes(self.socket.recv(2), byteorder="big")

        LOG.debug("<= %s %s", rdata.hex(), hex(sw)[2:])

        return sw, rdata

    def exchange(self, data: bytes) -> Tuple[int, bytes]:
        """Exchange (send + receive) with `self.socket`.

        Parameters
        ----------
        data : bytes
            Bytes with `data` to send.

        Returns
        -------
        Tuple[int, bytes]
            A pair (sw, rdata) containing the status word and response data.

        """
        self.send(data)

        return self.recv()  # blocking IO

    def close(self) -> None:
        """Close connection to TCP socket `self.socket`.

        Returns
        -------
        None

        """
        if self.__opened:
            self.socket.close()
            self.__opened = False



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class PerformAttestationRequest(google.protobuf.message.Message):
    """Deprecated, last used in v1.0.0"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHALLENGE_FIELD_NUMBER: builtins.int
    challenge: builtins.bytes
    """32 bytes challenge."""
    def __init__(
        self,
        *,
        challenge: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["challenge", b"challenge"]) -> None: ...

global___PerformAttestationRequest = PerformAttestationRequest

@typing.final
class PerformAttestationResponse(google.protobuf.message.Message):
    """Deprecated, last used in v1.0.0"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BOOTLOADER_HASH_FIELD_NUMBER: builtins.int
    DEVICE_PUBKEY_FIELD_NUMBER: builtins.int
    CERTIFICATE_FIELD_NUMBER: builtins.int
    ROOT_PUBKEY_IDENTIFIER_FIELD_NUMBER: builtins.int
    CHALLENGE_SIGNATURE_FIELD_NUMBER: builtins.int
    bootloader_hash: builtins.bytes
    device_pubkey: builtins.bytes
    certificate: builtins.bytes
    root_pubkey_identifier: builtins.bytes
    challenge_signature: builtins.bytes
    def __init__(
        self,
        *,
        bootloader_hash: builtins.bytes = ...,
        device_pubkey: builtins.bytes = ...,
        certificate: builtins.bytes = ...,
        root_pubkey_identifier: builtins.bytes = ...,
        challenge_signature: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["bootloader_hash", b"bootloader_hash", "certificate", b"certificate", "challenge_signature", b"challenge_signature", "device_pubkey", b"device_pubkey", "root_pubkey_identifier", b"root_pubkey_identifier"]) -> None: ...

global___PerformAttestationResponse = PerformAttestationResponse



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mnemonic.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0emnemonic.proto\x12\x14shiftcrypto.bitbox02\"\x15\n\x13ShowMnemonicRequest\"H\n\x1aRestoreFromMnemonicRequest\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\x17\n\x0ftimezone_offset\x18\x02 \x01(\x05\"6\n#SetMnemonicPassphraseEnabledRequest\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mnemonic_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SHOWMNEMONICREQUEST._serialized_start=40
  _SHOWMNEMONICREQUEST._serialized_end=61
  _RESTOREFROMMNEMONICREQUEST._serialized_start=63
  _RESTOREFROMMNEMONICREQUEST._serialized_end=135
  _SETMNEMONICPASSPHRASEENABLEDREQUEST._serialized_start=137
  _SETMNEMONICPASSPHRASEENABLEDREQUEST._serialized_end=191
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: bitbox02_system.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x62itbox02_system.proto\x12\x14shiftcrypto.bitbox02\"\x14\n\x12\x43heckSDCardRequest\"\'\n\x13\x43heckSDCardResponse\x12\x10\n\x08inserted\x18\x01 \x01(\x08\"\x13\n\x11\x44\x65viceInfoRequest\"\xd8\x02\n\x12\x44\x65viceInfoResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0binitialized\x18\x02 \x01(\x08\x12\x0f\n\x07version\x18\x03 \x01(\t\x12#\n\x1bmnemonic_passphrase_enabled\x18\x04 \x01(\x08\x12&\n\x1emonotonic_increments_remaining\x18\x05 \x01(\r\x12\x18\n\x10securechip_model\x18\x06 \x01(\t\x12J\n\tbluetooth\x18\x07 \x01(\x0b\x32\x32.shiftcrypto.bitbox02.DeviceInfoResponse.BluetoothH\x00\x88\x01\x01\x1aM\n\tBluetooth\x12\x15\n\rfirmware_hash\x18\x01 \x01(\x0c\x12\x18\n\x10\x66irmware_version\x18\x02 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x03 \x01(\x08\x42\x0c\n\n_bluetooth\"\x9b\x01\n\x19InsertRemoveSDCardRequest\x12L\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32<.shiftcrypto.bitbox02.InsertRemoveSDCardRequest.SDCardAction\"0\n\x0cSDCardAction\x12\x0f\n\x0bREMOVE_CARD\x10\x00\x12\x0f\n\x0bINSERT_CARD\x10\x01\"\x0e\n\x0cResetRequest\",\n\x18SetDeviceLanguageRequest\x12\x10\n\x08language\x18\x01 \x01(\t\"$\n\x14SetDeviceNameRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"%\n\x12SetPasswordRequest\x12\x0f\n\x07\x65ntropy\x18\x01 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bitbox02_system_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CHECKSDCARDREQUEST._serialized_start=47
  _CHECKSDCARDREQUEST._serialized_end=67
  _CHECKSDCARDRESPONSE._serialized_start=69
  _CHECKSDCARDRESPONSE._serialized_end=108
  _DEVICEINFOREQUEST._serialized_start=110
  _DEVICEINFOREQUEST._serialized_end=129
  _DEVICEINFORESPONSE._serialized_start=132
  _DEVICEINFORESPONSE._serialized_end=476
  _DEVICEINFORESPONSE_BLUETOOTH._serialized_start=385
  _DEVICEINFORESPONSE_BLUETOOTH._serialized_end=462
  _INSERTREMOVESDCARDREQUEST._serialized_start=479
  _INSERTREMOVESDCARDREQUEST._serialized_end=634
  _INSERTREMOVESDCARDREQUEST_SDCARDACTION._serialized_start=586
  _INSERTREMOVESDCARDREQUEST_SDCARDACTION._serialized_end=634
  _RESETREQUEST._serialized_start=636
  _RESETREQUEST._serialized_end=650
  _SETDEVICELANGUAGEREQUEST._serialized_start=652
  _SETDEVICELANGUAGEREQUEST._serialized_end=696
  _SETDEVICENAMEREQUEST._serialized_start=698
  _SETDEVICENAMEREQUEST._serialized_end=734
  _SETPASSWORDREQUEST._serialized_start=736
  _SETPASSWORDREQUEST._serialized_end=773
# @@protoc_insertion_point(module_scope)




# Our simple firmware header.
# Although called a header, this data is placed into the middle of the binary.
# It is located at start of firmware + 16k - sizeof(heaer). This is a gap unused in normal
# micropython layout. Exactly the last 64 bytes (signature) should be left out of
# the checksum. We do checksum areas beyond the end of the last byte of firmware (up to length)
# and expect those regions to be unprogrammed flash (ones).
# - timestamp must increase with each upgrade (downgrade protection)
# - version_string is for humans only
# - pubkey_num indicates which pubkey was used for signature
# - firmware_length, must be:
#   - bigger than minimum length, less than max
#   - 512-byte aligned
# - bootloader assumes the flash filesystem (FAT FS) follows the firmware.
# - timestamp is YYMMDDHHMMSS0000 in BCD


FW_HEADER_SIZE = 128
FW_HEADER_OFFSET = (0x4000-FW_HEADER_SIZE)

FW_HEADER_MAGIC = 0xCC001234

# Firmware Image Size

# arbitrary min size
FW_MIN_LENGTH = (256*1024)

# (mk1-3) absolute max size: 1MB flash - 32k for bootloader
# practical limit for our-protocol USB upgrades: 786432 (or else settings damaged)
FW_MAX_LENGTH = (0x100000 - 0x8000)

# .. for Mk4: 2Mbytes, less bootrom of 128k.
FW_MAX_LENGTH_MK4 = (0x200000 - 0x20000)

# Arguments to be used w/ python's struct module.
FWH_PY_FORMAT = "<I8s8sIIII8s20s64s"
FWH_PY_VALUES = "magic_value timestamp version_string pubkey_num firmware_length install_flags hw_compat best_ts future signature"
FWH_NUM_FUTURE = 7

# offset of pubkey number
FWH_PK_NUM_OFFSET = 20

# Bits in install_flags
FWHIF_HIGH_WATER = 0x01
FWHIF_BEST_TS = 0x02

# Bits in hw_compat
MK_1_OK = 0x01
MK_2_OK = 0x02
MK_3_OK = 0x04
MK_4_OK = 0x08
MK_Q1_OK = 0x10
# RFU:
MK_6_OK = 0x20

# EOF


"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2019 Shift Cryptosecurity AG

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CheckSDCardRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CheckSDCardRequest = CheckSDCardRequest

@typing.final
class CheckSDCardResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INSERTED_FIELD_NUMBER: builtins.int
    inserted: builtins.bool
    def __init__(
        self,
        *,
        inserted: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["inserted", b"inserted"]) -> None: ...

global___CheckSDCardResponse = CheckSDCardResponse

@typing.final
class DeviceInfoRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeviceInfoRequest = DeviceInfoRequest

@typing.final
class DeviceInfoResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Bluetooth(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FIRMWARE_HASH_FIELD_NUMBER: builtins.int
        FIRMWARE_VERSION_FIELD_NUMBER: builtins.int
        ENABLED_FIELD_NUMBER: builtins.int
        firmware_hash: builtins.bytes
        """Hash of the currently active Bluetooth firmware on the device."""
        firmware_version: builtins.str
        """Firmware version, formated as "major.minor.patch"."""
        enabled: builtins.bool
        """True if Bluetooth is enabled"""
        def __init__(
            self,
            *,
            firmware_hash: builtins.bytes = ...,
            firmware_version: builtins.str = ...,
            enabled: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["enabled", b"enabled", "firmware_hash", b"firmware_hash", "firmware_version", b"firmware_version"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    INITIALIZED_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    MNEMONIC_PASSPHRASE_ENABLED_FIELD_NUMBER: builtins.int
    MONOTONIC_INCREMENTS_REMAINING_FIELD_NUMBER: builtins.int
    SECURECHIP_MODEL_FIELD_NUMBER: builtins.int
    BLUETOOTH_FIELD_NUMBER: builtins.int
    name: builtins.str
    initialized: builtins.bool
    version: builtins.str
    mnemonic_passphrase_enabled: builtins.bool
    monotonic_increments_remaining: builtins.int
    securechip_model: builtins.str
    """From v9.6.0: "ATECC608A" or "ATECC608B"."""
    @property
    def bluetooth(self) -> global___DeviceInfoResponse.Bluetooth:
        """Only present in Bluetooth-enabled devices."""

    def __init__(
        self,
        *,
        name: builtins.str = ...,
        initialized: builtins.bool = ...,
        version: builtins.str = ...,
        mnemonic_passphrase_enabled: builtins.bool = ...,
        monotonic_increments_remaining: builtins.int = ...,
        securechip_model: builtins.str = ...,
        bluetooth: global___DeviceInfoResponse.Bluetooth | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_bluetooth", b"_bluetooth", "bluetooth", b"bluetooth"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_bluetooth", b"_bluetooth", "bluetooth", b"bluetooth", "initialized", b"initialized", "mnemonic_passphrase_enabled", b"mnemonic_passphrase_enabled", "monotonic_increments_remaining", b"monotonic_increments_remaining", "name", b"name", "securechip_model", b"securechip_model", "version", b"version"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["_bluetooth", b"_bluetooth"]) -> typing.Literal["bluetooth"] | None: ...

global___DeviceInfoResponse = DeviceInfoResponse

@typing.final
class InsertRemoveSDCardRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _SDCardAction:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _SDCardActionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[InsertRemoveSDCardRequest._SDCardAction.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        REMOVE_CARD: InsertRemoveSDCardRequest._SDCardAction.ValueType  # 0
        INSERT_CARD: InsertRemoveSDCardRequest._SDCardAction.ValueType  # 1

    class SDCardAction(_SDCardAction, metaclass=_SDCardActionEnumTypeWrapper): ...
    REMOVE_CARD: InsertRemoveSDCardRequest.SDCardAction.ValueType  # 0
    INSERT_CARD: InsertRemoveSDCardRequest.SDCardAction.ValueType  # 1

    ACTION_FIELD_NUMBER: builtins.int
    action: global___InsertRemoveSDCardRequest.SDCardAction.ValueType
    def __init__(
        self,
        *,
        action: global___InsertRemoveSDCardRequest.SDCardAction.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["action", b"action"]) -> None: ...

global___InsertRemoveSDCardRequest = InsertRemoveSDCardRequest

@typing.final
class ResetRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ResetRequest = ResetRequest

@typing.final
class SetDeviceLanguageRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LANGUAGE_FIELD_NUMBER: builtins.int
    language: builtins.str
    def __init__(
        self,
        *,
        language: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["language", b"language"]) -> None: ...

global___SetDeviceLanguageRequest = SetDeviceLanguageRequest

@typing.final
class SetDeviceNameRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name"]) -> None: ...

global___SetDeviceNameRequest = SetDeviceNameRequest

@typing.final
class SetPasswordRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTROPY_FIELD_NUMBER: builtins.int
    entropy: builtins.bytes
    def __init__(
        self,
        *,
        entropy: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["entropy", b"entropy"]) -> None: ...

global___SetPasswordRequest = SetPasswordRequest



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2019 Shift Cryptosecurity AG

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ShowMnemonicRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ShowMnemonicRequest = ShowMnemonicRequest

@typing.final
class RestoreFromMnemonicRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMEZONE_OFFSET_FIELD_NUMBER: builtins.int
    timestamp: builtins.int
    timezone_offset: builtins.int
    def __init__(
        self,
        *,
        timestamp: builtins.int = ...,
        timezone_offset: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["timestamp", b"timestamp", "timezone_offset", b"timezone_offset"]) -> None: ...

global___RestoreFromMnemonicRequest = RestoreFromMnemonicRequest

@typing.final
class SetMnemonicPassphraseEnabledRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLED_FIELD_NUMBER: builtins.int
    enabled: builtins.bool
    def __init__(
        self,
        *,
        enabled: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["enabled", b"enabled"]) -> None: ...

global___SetMnemonicPassphraseEnabledRequest = SetMnemonicPassphraseEnabledRequest
